import math
from io import BytesIO
import collections
from larry import s3


def scale_image_to_size(image=None, bucket=None, key=None, uri=None, max_pixels=None, max_bytes=None):
    try:
        from PIL import Image
        if image:
            src_bytes = _image_byte_count(image)
        else:
            src_bytes = s3.get_object_size(bucket, key, uri)
            image = s3.read_pillow_image(bucket, key, uri)
        x, y = image.size
        src_pixels = x * y
        bytes_scalar = math.sqrt(max_bytes/src_bytes) if max_bytes else 1
        pixels_scalar = math.sqrt(max_pixels/src_pixels) if max_pixels else 1
        scalar = min(bytes_scalar, pixels_scalar)
        if scalar >= 1:
            return image, None
        else:
            new_x = int(scalar * x)
            new_y = int(scalar * y)
            new_image, scalar = image.resize((new_x, new_y), Image.BICUBIC), scalar
            if max_bytes and _image_byte_count(new_image) > max_bytes:
                return scale_image_to_size(image=image, max_pixels=max_pixels, max_bytes=int(max_bytes*0.95))
            else:
                return new_image, scalar
    except ImportError as e:
        # Simply raise the ImportError to let the user know this requires Pillow to function
        raise e


def _image_byte_count(image):
    buff = BytesIO()
    image.save(buff, 'PNG' if image.format is None else image.format)
    return len(buff.getvalue())


def annotation_to_coordinates(box):
    return [box['left'], box['top'], box['left']+box['width']-1, box['top']+box['height']-1]


def box_coordinates(box):
    coords = box.get('coordinates')
    if coords:
        return coords
    else:
        return annotation_to_coordinates(box)


def scale_box(box, scalar):
    bbox = augment_box_attributes(box.copy())
    bbox['left'] = box['left'] * scalar
    bbox['top'] = box['top'] * scalar
    bbox['width'] = box['width'] * scalar
    bbox['height'] = box['height'] * scalar
    bbox['coordinates'] = annotation_to_coordinates(bbox)
    return bbox


def augment_box_attributes(box):
    if 'coordinates' not in box and 'top' in box and 'left' in box and 'width' in box and 'height' in box:
        box['coordinates'] = annotation_to_coordinates(box)
    elif 'coordinates' in box and ('top' not in box or 'left' not in box):
        box['left'] = box['coordinates'][0]
        box['top'] = box['coordinates'][1]
        box['width'] = box['coordinates'][2] - box['coordinates'][0] + 1
        box['height'] = box['coordinates'][3] - box['coordinates'][1] + 1
    return box


def box_area(box):
    if isinstance(box, collections.Mapping):
        box = box_coordinates(box)
    if box:
        return (box[2] - box[0]) * (box[3] - box[1])
    else:
        return 0


def box_intersection(a, b):
    if isinstance(a, collections.Mapping):
        a = box_coordinates(a)
    if isinstance(b, collections.Mapping):
        b = box_coordinates(b)
    intersection = max(a[0], b[0]), max(a[1], b[1]), min(a[2], b[2]), min(a[3], b[3])
    if intersection[2] < intersection[0] or intersection[3] < intersection[1]:
        return None
    else:
        return intersection


def intersection_over_union(a, b):
    ac = box_coordinates(a)
    bc = box_coordinates(b)
    intersection = box_area(box_intersection(ac, bc))
    union = box_area(ac) + box_area(bc) - intersection
    return intersection / union


def render_boxes(boxes,
                 image=None,
                 image_uri=None,
                 color=None,
                 width=None,
                 label_size=None,
                 label=None,
                 annotation_filter=None,
                 get_box=None,
                 color_index=None):
    if image_uri:
        image = s3.read_pillow_image(uri=image_uri)
    # Change palette mode images to RGB so that standard palette colors can be drawn on them
    if image.mode == 'P':
        image = image.convert(mode='RGB')
    else:
        image = image.copy()
    if color is None:
        color = get_color_list()
    width = round(image.width / 512) + 1 if width is None else width
    label_size = 20 if label_size is None else label_size

    try:
        from PIL import ImageDraw, ImageFont
        draw = ImageDraw.Draw(image)

        # TODO Find a better way to pull in fonts
        font = ImageFont.truetype("/usr/share/fonts/dejavu/DejaVuSans.ttf", size=label_size)

        for idx, item in enumerate(boxes):
            if annotation_filter is None or annotation_filter(idx, item):

                # Retrieve the box coordinates from the annotation
                if get_box:
                    item = get_box(idx, item)
                box = box_coordinates(item)

                # select a color to use with this box
                if isinstance(color, str):
                    box_color = color
                elif callable(color):
                    box_color = color(idx, item)
                elif color_index:
                    # TODO: Fix array out of bounds issue
                    box_color = color[color_index(idx, item)]
                else:
                    box_color = 'green'
                draw.rectangle(box, outline=box_color, width=width)
                if label:
                    if isinstance(label, str):
                        text = label
                    else:
                        text = label(idx, item)
                    size = draw.textsize(text,
                                         font=font,
                                         spacing=0)
                    draw.multiline_text((box[0] - size[0] - 4, box[1] + 4),
                                        text,
                                        fill=box_color,
                                        font=font,
                                        spacing=0,
                                        align='right')
        return image
    except ImportError as e:
        # Simply raise the ImportError to let the user know this requires Pillow to function
        raise e


def get_color_list():
    return [(44, 160, 44), (31, 119, 180), (255, 127, 14), (214, 39, 40), (148, 103, 189), (140, 86, 75),
            (227, 119, 194), (127, 127, 127), (188, 189, 34), (255, 152, 150), (23, 190, 207),
            (174, 199, 232), (255, 187, 120), (152, 223, 138), (197, 176, 213)]


def generate_label(keys):
    key_list = keys

    def gen_label(idx, item):
        label = ''
        for key in key_list:
            if len(label) != 0:
                label = label + '\n'
            if key == 'index':
                label = label + item.get(key, idx)
            else:
                label = item.get(key)
        return label


# TODO: Add image_url parameter
def render_bounding_box_assignments(assignments,
                                    image=None,
                                    image_uri=None,
                                    labels=None,
                                    single_image=True):
    return render_boxes_from_objects(assignments,
                                     lambda item: item['Answer']['boundingBox']['boundingBoxes'],
                                     image,
                                     image_uri,
                                     labels,
                                     single_image)


def render_boxes_from_objects(objects,
                              accessor,
                              image=None,
                              image_uri=None,
                              labels=None,
                              single_image=True,
                              color=None):
    # TODO: Avoid an unnecessary copy step when coming from uri
    if image_uri:
        image = s3.read_pillow_image(uri=image_uri)
    # Change palette mode images to RGB so that standard palette colors can be drawn on them
    if image.mode == 'P':
        image = image.convert(mode='RGB')
    else:
        image = image.copy()

    if isinstance(objects, collections.Mapping):
        annotation = accessor(objects)
        if labels:
            return render_boxes(annotation, image=image, image_uri=image_uri, color=color,
                                color_index=lambda idx, item: _find_label_index(item, labels))
        else:
            return render_boxes(annotation, image=image, image_uri=image_uri, color=color)
    elif single_image:
        for idx, obj in enumerate(objects):
            annotation = accessor(obj)
            if callable(color):
                image = render_boxes(annotation, image=image,
                                     color=lambda o_idx, e_idx, item: color(idx, e_idx, item),
                                     color_index=lambda i, item: idx)
            else:
                image = render_boxes(annotation, image=image,
                                     color_index=lambda i, item: idx)
        return image
    else:
        images = []
        for idx, obj in enumerate(objects):
            annotation = accessor(obj)
            if callable(color):
                images.append(render_boxes(annotation, image=image,
                                           color=lambda e_idx, item: color(idx, e_idx, item),
                                           color_index=lambda i, item: idx))
            else:
                images.append(render_boxes(annotation, image=image,
                                           color_index=lambda i, item: idx))
        return images


def _find_label_index(item, labels):
    if 'label' in item:
        for idx, label in enumerate(labels):
            if label == item['label']:
                return idx
    else:
        return 0


def tile_images(images, max_width=3000):
    try:
        from PIL import Image

        # TODO: Add handling for variably sized images?
        (x, y) = images[0].size
        columns = min(max(math.floor(max_width / x), 1), len(images))
        rows = math.ceil(len(images) / columns)
        canvas = Image.new('RGB', (x * columns, y * rows), color='white')

        for idx, image in enumerate(images):
            row = math.floor(idx / columns)
            column = idx - (row * columns)
            canvas.paste(image, (column * x, row * y))
        return canvas
    except ImportError as e:
        # Simply raise the ImportError to let the user know this requires Pillow to function
        raise e


def join_images(images, horizontal=True):
    try:
        from PIL import Image

        width = 0
        height = 0
        indices = []

        if horizontal:
            for image in images:
                (x, y) = image.size
                height = max(height, y)
                indices.append(width)
                width += x
            indices = [(x, 0) for x in indices]
        else:
            for image in images:
                (x, y) = image.size
                width = max(width, x)
                indices.append(height)
                height += y
            indices = [(0, y) for y in indices]

        canvas = Image.new('RGB', (width, height), color='white')
        for idx, image in enumerate(images):
            canvas.paste(image, indices[idx])

        return canvas, indices
    except ImportError as e:
        # Simply raise the ImportError to let the user know this requires Pillow to function
        raise e



import boto3
import botocore
from botocore.exceptions import ClientError
from boto3.s3.transfer import TransferConfig
from io import StringIO, BytesIO
import os
import json
from larry import utils
from larry import sts
from larry import types
import uuid
from urllib.request import Request
from urllib import parse
from urllib import request
from zipfile import ZipFile
from collections import Mapping
import inspect
import re

# Local S3 resource object
resource = None
# A local instance of the boto3 session to use
__session = boto3.session.Session()

ACL_PRIVATE = 'private'
ACL_PUBLIC_READ = 'public-read'
ACL_PUBLIC_READ_WRITE = 'public-read-write'
ACL_AUTHENTICATED_READ = 'authenticated-read'
ACL_AWS_EXEC_READ = 'aws-exec-read'
ACL_BUCKET_OWNER_READ = 'bucket-owner-read'
ACL_BUCKET_OWNER_FULL_CONTROL = 'bucket-owner-full-control'

CLASS_STANDARD = 'STANDARD'
CLASS_REDUCED_REDUNDANCY = 'REDUCED_REDUNDANCY'
CLASS_STANDARD_IA = 'STANDARD_IA'
CLASS_ONEZONE_IA = 'ONEZONE_IA'
CLASS_INTELLIGENT_TIERING = 'INTELLIGENT_TIERING'
CLASS_GLACIER = 'GLACIER'
CLASS_DEEP_ARCHIVE = 'DEEP_ARCHIVE'


def set_session(aws_access_key_id=None,
                aws_secret_access_key=None,
                aws__session_token=None,
                region_name=None,
                profile_name=None,
                boto_session=None):
    """
    Sets the boto3 session for this module to use a specified configuration state.
    :param aws_access_key_id: AWS access key ID
    :param aws_secret_access_key: AWS secret access key
    :param aws__session_token: AWS temporary session token
    :param region_name: Default region when creating new connections
    :param profile_name: The name of a profile to use
    :param boto_session: An existing session to use
    :return: None
    """
    global __session, resource
    __session = boto_session if boto_session is not None else boto3.session.Session(
        **utils.copy_non_null_keys(locals()))
    sts.set_session(boto_session=__session)
    resource = __session.resource('s3')


def __load_resource(func):
    def decorated(*args, **kwargs):
        if 's3_resource' not in kwargs:
            kwargs['s3_resource'] = resource
        return func(*args, **kwargs)

    return decorated


def __decompose_location(require_bucket=True, require_key=False, key_arg='key', allow_multiple=False):
    def decorate(func):
        spec = inspect.getfullargspec(func)
        offset = len(spec.args)

        def decomposed(*args, **kwargs):
            location = args[offset:]
            uri, bucket, key = (None, None, None)
            if kwargs.get('uri') is None and kwargs.get('bucket') is None and kwargs.get(key_arg) is None:
                if len(location) == 0:
                    raise Exception('A location must be specified')
                if len(location) > 2:
                    raise Exception('Too many location values')
                if len(location) == 1:
                    if isinstance(location[0], list) and location[0][0].startswith('s3:'):
                        uri = location[0]
                    elif isinstance(location[0], str) and location[0].startswith('s3:'):
                        uri = location[0]
                    else:
                        bucket = location[0]
                else:
                    (bucket, key) = location
            elif len(location) > 0:
                raise Exception('Both positional location and ' + key_arg + ' values are present')

            if 'uri' in kwargs and kwargs['uri'] is not None:
                uri = kwargs['uri']
            if 'bucket' in kwargs and kwargs['bucket'] is not None:
                bucket = kwargs['bucket']
            if key_arg in kwargs and kwargs[key_arg] is not None:
                key = kwargs[key_arg]

            if uri:
                if isinstance(uri, list) and len(uri) > 0:
                    if not allow_multiple:
                        raise Exception('You cannot provide a list of URIs for {}'.format(func.__name__))
                    pairs = [decompose_uri(u) for u in uri]
                    key = []
                    bucket = pairs[0][0]
                    for pair in pairs:
                        key.append(pair[1])
                        if pair[0] != bucket:
                            raise Exception('Multiple values for bucket are not allowed')
                else:
                    (bucket, key) = decompose_uri(uri)
                    if bucket is None:
                        raise Exception('Invalid S3 URI')

            if isinstance(key, list) and not allow_multiple:
                raise Exception('You cannot provide a list of keys for {}'.format(func.__name__))

            if require_bucket and (bucket is None or len(bucket) == 0):
                raise Exception('A bucket must be provided')
            if require_key and (key is None or len(key) == 0):
                raise Exception('A key must be provided')

            kwargs['bucket'] = bucket
            kwargs[key_arg] = key
            return func(*args, **kwargs)

        return decomposed

    return decorate


@__load_resource
@__decompose_location(require_key=True)
def obj(*location, bucket=None, key=None, uri=None, s3_resource=None):
    """
    Retrieves header information for an object from S3 and returns it in a boto3 Object object.
    :param location: Positional values for bucket, key, and/or uri
    :param bucket: The S3 bucket
    :param key: The key of the object
    :param uri: An s3:// path containing the bucket and key of the object
    :param s3_resource: Boto3 resource to use if you don't wish to use the default resource
    :return: Boto3 S3 Object
    """
    o = s3_resource.Bucket(bucket).Object(key=key)
    return o


@__load_resource
@__decompose_location(require_key=True, allow_multiple=True)
def delete(*location, bucket=None, key=None, uri=None, s3_resource=None):
    """
    Deletes the object defined by the bucket/key pair or uri.
    :param location: Positional values for bucket, key, and/or uri
    :param bucket: The S3 bucket
    :param key: The key of the object
    :param uri: An s3:// path containing the bucket and key of the object
    :param s3_resource: Boto3 resource to use if you don't wish to use the default resource
    :return: Dict containing the boto3 response
    """
    if isinstance(key, list):
        return s3_resource.Bucket(bucket).delete_objects(Delete={'Objects': [{'Key': k} for k in key], 'Quiet': True})
    else:
        return s3_resource.Bucket(bucket).Object(key=key).delete()


@__load_resource
@__decompose_location(require_key=True)
def get(*location, bucket=None, key=None, uri=None, s3_resource=None):
    """
    Performs a 'get' of the object defined by the bucket/key pair or uri.
    :param location: Positional values for bucket, key, and/or uri
    :param bucket: The S3 bucket for object to retrieve
    :param key: The key of the object to be retrieved from the bucket
    :param uri: An s3:// path containing the bucket and key of the object
    :param s3_resource: Boto3 resource to use if you don't wish to use the default resource
    :return: Dict containing the Body of the object and associated attributes
    """
    return obj(bucket=bucket, key=key, s3_resource=s3_resource).get()


@__load_resource
@__decompose_location(require_key=True)
def get_size(*location, bucket=None, key=None, uri=None, s3_resource=None):
    """
    Returns the content_length of an S3 object.
    :param location: Positional values for bucket, key, and/or uri
    :param bucket: The S3 bucket for object to retrieve
    :param key: The key of the object to be retrieved from the bucket
    :param uri: An s3:// path containing the bucket and key of the object
    :param s3_resource: Boto3 resource to use if you don't wish to use the default resource
    :return: Size in bytes
    """
    return obj(bucket=bucket, key=key, s3_resource=s3_resource).content_length


@__load_resource
@__decompose_location(require_key=True)
def read(*location, bucket=None, key=None, uri=None, byte_count=None, s3_resource=None):
    """
    Performs a 'get' of the object defined by the bucket/key pair or uri
    and then performs a 'read' of the Body of that object.
    :param location: Positional values for bucket, key, and/or uri
    :param bucket: The S3 bucket for object to retrieve
    :param key: The key of the object to be retrieved from the bucket
    :param uri: An s3:// path containing the bucket and key of the object
    :param byte_count: The max number of bytes to read from the object. All data is read if omitted.
    :param s3_resource: Boto3 resource to use if you don't wish to use the default resource
    :return: The bytes contained in the object
    """
    return get(bucket=bucket, key=key, uri=uri, s3_resource=s3_resource)['Body'].read(byte_count)


@__load_resource
@__decompose_location(require_key=True)
def read_as(o_type, *location, bucket=None, key=None, uri=None, encoding='utf-8', s3_resource=None):
    """
    Reads in the s3 object defined by the bucket/key pair or uri and loads the
    contents into an object of the specified type
    :param o_type: A value defined in larry.types to load the data using
    :param location: Positional values for bucket, key, and/or uri
    :param bucket: The S3 bucket for object to retrieve
    :param key: The key of the object to be retrieved from the bucket
    :param uri: An s3:// path containing the bucket and key of the object
    :param encoding: The charset to use when decoding the object bytes, utf-8 by default
    :param s3_resource: Boto3 resource to use if you don't wish to use the default resource
    :return: An object representation of the data in S3
    """
    objct = read(bucket=bucket, key=key, uri=uri, s3_resource=s3_resource)
    # TODO: Would a handler or local constant be a better idea here?
    if o_type == types.TYPE_DICT:
        return json.loads(objct.decode(encoding), object_hook=utils.JSONDecoder)
    elif o_type == types.TYPE_STRING:
        return objct.decode(encoding)
    elif o_type == types.TYPE_PILLOW_IMAGE:
        try:
            from PIL import Image
            return Image.open(BytesIO(objct))
        except ImportError as e:
            # Simply raise the ImportError to let the user know this requires Pillow to function
            raise e
    else:
        raise Exception('Unhandled type')


@__load_resource
@__decompose_location(require_key=True)
def read_list_as(o_type, *location, bucket=None, key=None, uri=None, encoding='utf-8', newline='\n', s3_resource=None):
    """
    Reads in the s3 object defined by the bucket/key pair or uri, decodes it to a string, and
    splits it into lines. Returns an array containing an object representing the contents of each line.
    :param o_type: A value defined in larry.types to load the data using
    :param location: Positional values for bucket, key, and/or uri
    :param bucket: The S3 bucket for object to retrieve
    :param key: The key of the object to be retrieved from the bucket
    :param uri: An s3:// path containing the bucket and key of the object
    :param encoding: The charset to use when decoding the object bytes, utf-8 by default
    :param newline: The line separator to use when reading in the object, \n by default
    :param s3_resource: Boto3 resource to use if you don't wish to use the default resource
    :return: An object representation of the data in S3
    """
    objct = read(bucket=bucket, key=key, uri=uri, s3_resource=s3_resource)
    lines = objct.decode(encoding).split(newline)
    records = []
    for line in lines:
        if len(line) > 0:
            # TODO: Would a handler or local constant be a better idea here?
            if o_type == types.TYPE_DICT:
                records.append(json.loads(line, object_hook=utils.JSONDecoder))
            elif o_type == types.TYPE_STRING:
                records.append(line)
            else:
                raise Exception('Unhandled type')
    return records


@__load_resource
@__decompose_location(require_key=True)
def read_iter_as(o_type, *location, bucket=None, key=None, uri=None, encoding='utf-8', newline='\n', s3_resource=None):
    """
    Reads in the s3 object defined by the bucket/key pair or uri, decodes it to a string, and
    splits it into lines. Returns an iterator of objects representing the contents of each line.
    :param o_type: A value defined in larry.types to load the data using
    :param location: Positional values for bucket, key, and/or uri
    :param bucket: The S3 bucket for object to retrieve
    :param key: The key of the object to be retrieved from the bucket
    :param uri: An s3:// path containing the bucket and key of the object
    :param encoding: The charset to use when decoding the object bytes, utf-8 by default
    :param newline: The line separator to use when reading in the object, \n by default
    :param s3_resource: Boto3 resource to use if you don't wish to use the default resource
    :return: An object representation of the data in S3
    """
    objct = read(bucket=bucket, key=key, uri=uri, s3_resource=s3_resource)
    lines = objct.decode(encoding).split(newline)
    for line in lines:
        if len(line) > 0:
            # TODO: Would a handler or local constant be a better idea here?
            if o_type == types.TYPE_DICT:
                yield json.loads(line, object_hook=utils.JSONDecoder)
            elif o_type == types.TYPE_STRING:
                yield line
            else:
                raise Exception('Unhandled type')


@__load_resource
@__decompose_location(require_key=True)
def read_dict(*location, bucket=None, key=None, uri=None, encoding='utf-8', s3_resource=None):
    """
    Reads in the s3 object defined by the bucket/key pair or uri and
    loads the json contents into a dict.
    :param location: Positional values for bucket, key, and/or uri
    :param bucket: The S3 bucket for object to retrieve
    :param key: The key of the object to be retrieved from the bucket
    :param uri: An s3:// path containing the bucket and key of the object
    :param encoding: The charset to use when decoding the object bytes, utf-8 by default
    :param s3_resource: Boto3 resource to use if you don't wish to use the default resource
    :return: A dict representation of the json contained in the object
    """
    return read_as(types.TYPE_DICT, bucket=bucket, key=key, uri=uri, s3_resource=s3_resource)


@__load_resource
@__decompose_location(require_key=True)
def read_str(*location, bucket=None, key=None, uri=None, encoding='utf-8', s3_resource=None):
    """
    Reads in the s3 object defined by the bucket/key pair or uri and
    decodes it to text.
    :param location: Positional values for bucket, key, and/or uri
    :param bucket: The S3 bucket for object to retrieve
    :param key: The key of the object to be retrieved from the bucket
    :param uri: An s3:// path containing the bucket and key of the object
    :param encoding: The charset to use when decoding the object bytes, utf-8 by default
    :param s3_resource: Boto3 resource to use if you don't wish to use the default resource
    :return: The contents of the object as a string
    """
    return read_as(types.TYPE_STRING, bucket=bucket, key=key, uri=uri, s3_resource=s3_resource)


@__load_resource
@__decompose_location(require_key=True)
def read_list_of_dict(*location, bucket=None, key=None, uri=None, encoding='utf-8', newline='\n', s3_resource=None):
    """
    Reads in the s3 object defined by the bucket/key pair or uri and
    loads the JSON Lines data into a list of dict objects.
    :param location: Positional values for bucket, key, and/or uri
    :param bucket: The S3 bucket for object to retrieve
    :param key: The key of the object to be retrieved from the bucket
    :param uri: An s3:// path containing the bucket and key of the object
    :param encoding: The charset to use when decoding the object bytes, utf-8 by default
    :param newline: The line separator to use when reading in the object, \n by default
    :param s3_resource: Boto3 resource to use if you don't wish to use the default resource
    :return: The contents of the object as a list of dict objects
    """
    return read_list_as(types.TYPE_DICT, bucket=bucket, key=key, uri=uri,
                        encoding=encoding, newline=newline, s3_resource=s3_resource)


@__load_resource
@__decompose_location(require_key=True)
def read_list_of_str(*location, bucket=None, key=None, uri=None, encoding='utf-8', newline='\n', s3_resource=None):
    """
    Reads in the s3 object defined by the bucket/key pair or uri and
    loads the JSON Lines data into a list of dict objects.
    :param location: Positional values for bucket, key, and/or uri
    :param bucket: The S3 bucket for object to retrieve
    :param key: The key of the object to be retrieved from the bucket
    :param uri: An s3:// path containing the bucket and key of the object
    :param encoding: The charset to use when decoding the object bytes, utf-8 by default
    :param newline: The line separator to use when reading in the object, \n by default
    :param s3_resource: Boto3 resource to use if you don't wish to use the default resource
    :return: The contents of the object as a list of dict objects
    """
    return read_list_as(types.TYPE_STRING, bucket=bucket, key=key, uri=uri,
                        encoding=encoding, newline=newline, s3_resource=s3_resource)


@__load_resource
@__decompose_location(require_key=True)
def write(body, *location, bucket=None, key=None, uri=None, acl=None, content_type=None, content_encoding=None,
          content_language=None, content_length=None, metadata=None, sse=None, storage_class=None, redirect=None,
          tags=None, s3_resource=None):
    """
    Write an object to the bucket/key pair or uri.
    :param body: Data to write
    :param location: Positional values for bucket, key, and/or uri
    :param bucket: The S3 bucket for object to retrieve
    :param key: The key of the object to be retrieved from the bucket
    :param uri: An s3:// path containing the bucket and key of the object
    :param acl: The canned ACL to apply to the object
    :param content_type: A standard MIME type describing the format of the object data
    :param content_encoding: Specifies what content encodings have been applied to the object and thus what decoding
    mechanisms must be applied to obtain the media-type referenced by the Content-Type header field.
    :param content_language: The language the content is in.
    :param content_length: Size of the body in bytes.
    :param metadata: A map of metadata to store with the object in S3.
    :param sse: The server-side encryption algorithm used when storing this object in Amazon S3.
    :param storage_class: The S3 storage class to store the object in.
    :param redirect: If the bucket is configured as a website, redirects requests for this object to another object in
    the same bucket or to an external URL.
    :param tags: The tag-set for the object. Can be either a dict or url encoded key/value string.
    :param s3_resource: Boto3 resource to use if you don't wish to use the default resource
    :return: The URI of the object written to S3
    """
    params = utils.map_parameters(locals(), {
        'acl': 'ACL',
        'body': 'Body',
        'content_encoding': 'ContentEncoding',
        'content_language': 'ContentLanguage',
        'content_length': 'ContentLength',
        'content_type': 'ContentType',
        'metadata': 'Metadata',
        'sse': 'ServerSideEncryption',
        'storage_class': 'StorageClass',
        'redirect': 'WebsiteRedirectLocation'
    })
    if tags:
        params['Tagging'] = parse.urlencode(tags) if isinstance(tags, Mapping) else tags

    objct = s3_resource.Bucket(bucket).Object(key=key)
    objct.put(**params)
    return compose_uri(bucket, key)


extension_types = {
    'css': 'text/css',
    'html': 'text/html',
    'xhtml': 'text/html',
    'htm': 'text/html',
    'xml': 'text/xml',
    'csv': 'text/csv',
    'txt': 'text/plain',
    'png': 'image/png',
    'jpeg': 'image/jpeg',
    'jpg': 'image/jpeg',
    'gif': 'image/gif',
    'jsonl': 'application/x-jsonlines',
    'json': 'application/json',
    'js': 'application/javascript',
    'zip': 'application/zip',
    'pdf': 'application/pdf',
    'sql': 'application/sql'
}


@__load_resource
@__decompose_location(require_key=True)
def write_as(value, o_type, *location, bucket=None, key=None, uri=None, acl=None, newline='\n', delimiter=',',
             columns=None, headers=None, content_type=None, content_encoding=None, content_language=None,
             content_length=None, metadata=None, sse=None, storage_class=None, redirect=None, tags=None,
             s3_resource=None):
    """
    Write an object to the bucket/key pair (or uri), converting the python
    object to an appropriate format to write to file.
    :param value: Object to write to S3
    :param o_type: A value defined in larry.types to write the data using
    :param location: Positional values for bucket, key, and/or uri
    :param bucket: The S3 bucket for object to retrieve
    :param key: The key of the object to be retrieved from the bucket
    :param uri: An s3:// path containing the bucket and key of the object
    :param newline: Character(s) to use as a newline for list objects
    :param delimiter: Column delimiter to use, ',' by default
    :param columns: The columns to write out from the source rows, dict keys or list indexes
    :param headers: Headers to add to the output
    :param acl: The canned ACL to apply to the object
    :param content_type: Content type to apply to the file, if not present a suggested type will be applied
    :param content_encoding: Specifies what content encodings have been applied to the object and thus what decoding
    mechanisms must be applied to obtain the media-type referenced by the Content-Type header field.
    :param content_language: The language the content is in.
    :param content_length: Size of the body in bytes.
    :param metadata: A map of metadata to store with the object in S3.
    :param sse: The server-side encryption algorithm used when storing this object in Amazon S3.
    :param storage_class: The S3 storage class to store the object in.
    :param redirect: If the bucket is configured as a website, redirects requests for this object to another object in
    the same bucket or to an external URL.
    :param tags: The tag-set for the object. Can be either a dict or url encoded key/value string.
    :param s3_resource: Boto3 resource to use if you don't wish to use the default resource
    :return: The URI of the object written to S3
    """
    objct = None
    extension = key.split('.')[-1]
    if o_type == types.TYPE_STRING:
        if content_type is None:
            content_type = extension_types.get(extension, 'text/plain')
        objct = value
    elif o_type == types.TYPE_DICT:
        if content_type is None:
            content_type = 'application/json'
        objct = json.dumps(value, cls=utils.JSONEncoder)
    elif o_type == types.TYPE_PILLOW_IMAGE:
        objct = BytesIO()
        fmt = value.format if hasattr(value, 'format') and value.format is not None else 'PNG'
        value.save(objct, fmt)
        objct.seek(0)
        if content_type is None:
            content_type = extension_types.get(extension, extension_types.get(fmt.lower(), 'text/plain'))
    elif o_type == types.TYPE_JSON_LINES:
        if content_type is None:
            content_type = extension_types.get(extension, 'text/plain')
        buff = StringIO()
        for row in value:
            buff.write(json.dumps(row, cls=utils.JSONEncoder) + newline)
        objct = buff.getvalue()
    elif o_type == types.TYPE_DELIMITED:
        if content_type is None:
            content_type = extension_types.get(extension, 'text/plain')
        buff = StringIO()
        # empty
        if value is None or len(value) == 0:
            if headers:
                buff.write(_array_to_string(headers, delimiter) + newline)
            buff.write('')

        # list
        elif isinstance(value[0], list):
            indices = columns if columns else None
            if headers:
                buff.write(_array_to_string(headers, delimiter) + newline)
            for row in value:
                buff.write(_array_to_string(row, delimiter, indices) + newline)

        # dict
        elif isinstance(value[0], Mapping):
            keys = columns if columns else value[0].keys()
            buff.write(_array_to_string(headers if headers else keys, delimiter) + newline)

            for row in value:
                line = ''
                for i, k in enumerate(keys):
                    value = '' if row.get(k) is None else str(row.get(k))
                    line = value if i == 0 else line + delimiter + value
                buff.write(line + "\n")

        # string
        elif isinstance(value[0], str):
            buff.writelines(value)
        else:
            raise Exception('Invalid input')
        objct = buff.getvalue()
    else:
        raise Exception('Unhandled type')
    return write(objct, bucket=bucket, key=key, uri=uri, acl=acl, content_type=content_type,
                 content_encoding=content_encoding, content_language=content_language, content_length=content_length,
                 metadata=metadata, sse=sse, storage_class=storage_class, redirect=redirect, tags=tags,
                 s3_resource=s3_resource)


@__load_resource
@__decompose_location(require_key=True)
def write_object(value, *location, bucket=None, key=None, uri=None, newline='\n', acl=None, content_type=None,
                 content_encoding=None, content_language=None, content_length=None, metadata=None, sse=None,
                 storage_class=None, redirect=None, tags=None, s3_resource=None):
    """
    Write an object to the bucket/key pair (or uri), converting the python
    object to an appropriate format to write to file.
    :param value: Object to write to S3
    :param location: Positional values for bucket, key, and/or uri
    :param bucket: The S3 bucket for object to retrieve
    :param key: The key of the object to be retrieved from the bucket
    :param uri: An s3:// path containing the bucket and key of the object
    :param acl: The canned ACL to apply to the object
    :param newline: Character(s) to use as a newline for list objects
    :param content_type: Content type to apply to the file, if not present a suggested type will be applied
    :param content_encoding: Specifies what content encodings have been applied to the object and thus what decoding
    mechanisms must be applied to obtain the media-type referenced by the Content-Type header field.
    :param content_language: The language the content is in.
    :param content_length: Size of the body in bytes.
    :param metadata: A map of metadata to store with the object in S3.
    :param sse: The server-side encryption algorithm used when storing this object in Amazon S3.
    :param storage_class: The S3 storage class to store the object in.
    :param redirect: If the bucket is configured as a website, redirects requests for this object to another object in
    the same bucket or to an external URL.
    :param tags: The tag-set for the object. Can be either a dict or url encoded key/value string.
    :param s3_resource: Boto3 resource to use if you don't wish to use the default resource
    :return: The URI of the object written to S3
    """
    # TODO: Add support for pandas and numpy objects
    extension = key.split('.')[-1]
    # JSON
    if isinstance(value, Mapping):
        return write_as(value, types.TYPE_DICT, bucket=bucket, key=key, uri=uri, acl=acl, newline=newline,
                 content_type=content_type, content_encoding=content_encoding,
                 content_language=content_language, content_length=content_length, metadata=metadata, sse=sse,
                 storage_class=storage_class, redirect=redirect, tags=tags, s3_resource=s3_resource)
    # Text
    elif isinstance(value, str):
        return write_as(value, types.TYPE_STRING, bucket=bucket, key=key, uri=uri, acl=acl, newline=newline,
                 content_type=content_type, content_encoding=content_encoding,
                 content_language=content_language, content_length=content_length, metadata=metadata, sse=sse,
                 storage_class=storage_class, redirect=redirect, tags=tags, s3_resource=s3_resource)

    # List
    elif isinstance(value, list):
        if content_type is None:
            content_type = extension_types.get(extension, 'text/plain')
        buff = StringIO()
        for row in value:
            if isinstance(row, Mapping):
                buff.write(json.dumps(row, cls=utils.JSONEncoder) + newline)
            else:
                buff.write(str(row) + newline)
        return write(buff.getvalue(), bucket=bucket, key=key, uri=uri, acl=acl, content_type=content_type,
                     content_encoding=content_encoding, content_language=content_language,
                     content_length=content_length, metadata=metadata, sse=sse, storage_class=storage_class,
                     redirect=redirect, tags=tags, s3_resource=s3_resource)
    elif value is None:
        return write('', bucket=bucket, key=key, uri=uri, acl=acl, s3_resource=s3_resource, content_type=content_type,
                     content_encoding=content_encoding, content_language=content_language,
                     content_length=content_length, metadata=metadata, sse=sse, storage_class=storage_class,
                     redirect=redirect, tags=tags)
    else:
        # try to write it as an image
        # TODO: Rethink this, perhaps check for attributes first rather than try/fail
        try:
            return write_as(value, types.TYPE_PILLOW_IMAGE, bucket=bucket, key=key, uri=uri, acl=acl, newline=newline,
                     content_type=content_type, content_encoding=content_encoding, content_language=content_language,
                     content_length=content_length, metadata=metadata, sse=sse, storage_class=storage_class,
                     redirect=redirect, tags=tags, s3_resource=s3_resource)
        except Exception:
            return write(value, bucket=bucket, key=key, uri=uri, acl=acl, content_type=content_type,
                         content_encoding=content_encoding, content_language=content_language,
                         content_length=content_length, metadata=metadata, sse=sse, storage_class=storage_class,
                         redirect=redirect, tags=tags, s3_resource=s3_resource)


def _array_to_string(row, delimiter, indices=None):
    if indices is None:
        indices = range(len(row))
    line = ''
    for x in indices:
        line = str(row[x]) if x == 0 else line + delimiter + str(row[x])
    return line


@__load_resource
@__decompose_location(require_key=True)
def write_delimited(rows, *location, bucket=None, key=None, uri=None, acl=None, newline='\n', delimiter=',',
                    columns=None, headers=None, content_type=None, content_encoding=None, content_language=None,
                    content_length=None, metadata=None, sse=None, storage_class=None, redirect=None, tags=None,
                    s3_resource=None):
    """
    Write an object to the bucket/key pair (or uri), converting the python
    object to an appropriate format to write to file.
    :param rows: List of data to write, rows can be of type list, dict or str
    :param bucket: The S3 bucket for object to retrieve
    :param key: The key of the object to be retrieved from the bucket
    :param uri: An s3:// path containing the bucket and key of the object
    :param acl: The canned ACL to apply to the object
    :param newline: Character(s) to use as a newline for list objects
    :param delimiter: Column delimiter to use, ',' by default
    :param columns: The columns to write out from the source rows, dict keys or list indexes
    :param headers: Headers to add to the output
    :param content_type: Content type to apply to the file, if not present a suggested type will be applied
    :param content_encoding: Specifies what content encodings have been applied to the object and thus what decoding
    mechanisms must be applied to obtain the media-type referenced by the Content-Type header field.
    :param content_language: The language the content is in.
    :param content_length: Size of the body in bytes.
    :param metadata: A map of metadata to store with the object in S3.
    :param sse: The server-side encryption algorithm used when storing this object in Amazon S3.
    :param storage_class: The S3 storage class to store the object in.
    :param redirect: If the bucket is configured as a website, redirects requests for this object to another object in
    the same bucket or to an external URL.
    :param tags: The tag-set for the object. Can be either a dict or url encoded key/value string.
    :param s3_resource: Boto3 resource to use if you don't wish to use the default resource
    :return: The URI of the object written to S3
    """
    return write_as(rows, types.TYPE_DELIMITED, bucket=bucket, key=key, uri=uri, acl=acl, newline=newline,
                    delimiter=delimiter, columns=columns, headers=headers, content_type=content_type,
                    content_encoding=content_encoding, content_language=content_language, content_length=content_length,
                    metadata=metadata, sse=sse, storage_class=storage_class, redirect=redirect, tags=tags,
                    s3_resource=s3_resource)


@__load_resource
def rename(old_bucket=None, old_key=None, old_uri=None, new_bucket=None, new_key=None, new_uri=None,
           s3_resource=None):
    """
    Renames an object in S3.
    :param old_bucket: Source bucket
    :param old_key: Source key
    :param old_uri: An s3:// path containing the bucket and key of the source object
    :param new_bucket: Target bucket
    :param new_key: Target key
    :param new_uri: An s3:// path containing the bucket and key of the source object
    :param s3_resource: Boto3 resource to use if you don't wish to use the default resource
    :return: None
    """
    # TODO: Add support for passing location without parameter names
    if old_uri:
        (old_bucket, old_key) = decompose_uri(old_uri)
    if new_uri:
        (new_bucket, new_key) = decompose_uri(new_uri)
    s3_resource = s3_resource if s3_resource else resource
    copy_source = {
        'Bucket': old_bucket,
        'Key': old_key
    }
    s3_resource.meta.client.copy(copy_source, new_bucket, new_key)
    s3_resource.meta.client.delete_object(Bucket=old_bucket, Key=old_key)


@__load_resource
def copy(src_bucket=None, src_key=None, src_uri=None, new_bucket=None, new_key=None, new_uri=None,
         s3_resource=None):
    """
    Copies an object in S3.
    :param src_bucket: Source bucket
    :param src_key: Source key
    :param src_uri: An s3:// path containing the bucket and key of the source object
    :param new_bucket: Target bucket
    :param new_key: Target key
    :param new_uri: An s3:// path containing the bucket and key of the source object
    :param s3_resource: Boto3 resource to use if you don't wish to use the default resource
    :return: None
    """
    # TODO: Add support for passing location without parameter names
    if src_uri:
        (src_bucket, src_key) = decompose_uri(src_uri)
    if new_uri:
        (new_bucket, new_key) = decompose_uri(new_uri)
    s3_resource = s3_resource if s3_resource else resource
    s3_resource.meta.client.copy({'Bucket': src_bucket, 'Key': src_key}, new_bucket, new_key)


@__load_resource
@__decompose_location(require_key=True)
def exists(*location, bucket=None, key=None, uri=None, s3_resource=None):
    """
    Checks to see if an object with the given bucket/key (or uri) exists.
    :param bucket: The S3 bucket for the object
    :param key: The key of the object
    :param uri: An s3:// path containing the bucket and key of the object
    :param s3_resource: Boto3 resource to use if you don't wish to use the default resource
    :return: True if the key exists, if not, False
    """
    try:
        s3_resource.Object(bucket, key).load()
    except botocore.exceptions.ClientError as e:
        if e.response['Error']['Code'] == "404":
            return False
        else:
            raise e
    return True


uri_regex = re.compile("^[sS]3:\/\/([a-z0-9\.\-]{3,})\/?(.*)")


def decompose_uri(uri):
    """
    Decompose an S3 URI into a bucket and key
    :param uri: S3 URI
    :return: Tuple containing a bucket and key
    """
    m = uri_regex.match(uri)
    if m:
        return m.groups()
    else:
        return None, None


def get_bucket_name(uri):
    """
    Retrieve the bucket portion from an S3 URI
    :param uri: S3 URI
    :return: Bucket name
    """
    return decompose_uri(uri)[0]


def get_object_key(uri):
    """
    Retrieves the key portion of an S3 URI
    :param uri: S3 URI
    :return: Key value
    """
    return decompose_uri(uri)[1]


def compose_uri(bucket, key=None):
    """
    Compose a bucket and key into an S3 URI
    :param bucket: Bucket name
    :param key: Object key
    :return: S3 URI string
    """
    if key:
        return "s3://{}/{}".format(bucket, key)
    else:
        return "s3://{}/".format(bucket)


@__load_resource
@__decompose_location(key_arg='prefix')
def list_objects(*location, bucket=None, prefix=None, uri=None, include_empty_objects=False, s3_resource=None):
    """
    Returns a list of the object keys in the provided bucket that begin with the provided prefix.
    :param bucket: The S3 bucket to query
    :param prefix: The key prefix to use in searching the bucket
    :param uri: An s3:// path containing the bucket and prefix
    :param include_empty_objects: True if you want to include keys associated with objects of size=0
    :param s3_resource: Boto3 resource to use if you don't wish to use the default resource
    :return: A generator of object keys
    """
    paginator = s3_resource.meta.client.get_paginator('list_objects')
    operation_parameters = {'Bucket': bucket}
    if prefix:
        operation_parameters['Prefix'] = prefix
    page_iterator = paginator.paginate(**operation_parameters)
    for page in page_iterator:
        for objct in page.get('Contents', []):
            if objct['Size'] > 0 or include_empty_objects:
                yield objct['Key']


def _find_largest_common_prefix(values):
    """
    Searches through a list of values to find the longest possible common prefix amongst them. Useful for optimizing
    more costly searches. Supports lists of strings or tuples. If tuples are used, the first value is assumed to be
    the value to search on.
    :param values: List of values (strings or tuples containing a string in the first position)
    :return: String prefix common to all values
    """
    if isinstance(values[0], tuple):
        prefix, *_ = values[0]
    else:
        prefix = values[0]

    for value in values:
        key = value[0] if isinstance(value, tuple) else value
        while key[:len(prefix)] != prefix and len(prefix) > 0:
            prefix = prefix[:-1]
    return prefix


@__load_resource
def find_keys_not_present(bucket, keys=None, uris=None, s3_resource=None):
    """
    Searches an S3 bucket for a list of keys and returns any that cannot be found.
    :param bucket: The S3 bucket to search
    :param keys: A list of keys to search for (strings or tuples containing a string in the first position)
    :param uris: A list of S3 URIs to search for (strings or tuples containing a string in the first position)
    :param s3_resource: Boto3 resource to use if you don't wish to use the default resource
    :return: A list of keys that were not found (strings or tuples based on the input values)
    """

    # If URIs are passed, convert to a list of keys to use for the search
    if uris:
        keys = []
        for value in uris:
            if isinstance(value, tuple):
                uri, *z = value
                b, key = decompose_uri(uri)
                keys.append(tuple([key]) + tuple(z))
            else:
                b, key = decompose_uri(value)
                keys.append(key)

    # Find the longest common prefix to use as the search term
    prefix = _find_largest_common_prefix(keys)

    # Get a list of all keys in the bucket that match the prefix
    bucket_obj = s3_resource.Bucket(bucket)
    all_keys = []
    for objct in bucket_obj.objects.filter(Prefix=prefix):
        all_keys.append(objct.key)

    # Search for any keys that can't be found
    not_found = []
    for value in keys:
        key = value[0] if isinstance(value, tuple) else value
        if key not in all_keys:
            not_found.append(value)
    return not_found


@__load_resource
@__decompose_location(require_key=True)
def fetch(url, *location, bucket=None, key=None, uri=None, s3_resource=None, acl=None, **kwargs):
    """
    Retrieves the data defined by a URL to an S3 location.
    :param url: URL to retrieve
    :param bucket: The S3 bucket for the object
    :param key: The key of the object
    :param uri: An s3:// path containing the bucket and key of the object
    :param acl:
    :param s3_resource: Boto3 resource to use if you don't wish to use the default resource
    :return: The URI of the object written to S3
    """
    req = Request(url, **kwargs)
    with request.urlopen(req) as response:
        return write(response.read(), bucket=bucket, key=key, s3_resource=s3_resource, acl=acl)


@__load_resource
@__decompose_location(require_key=True)
def download(directory, *location, bucket=None, key=None, uri=None, use_threads=True, s3_resource=None):
    """
    Downloads the an S3 object to a directory on the local file system.
    :param directory: The directory to download the object to
    :param bucket: The S3 bucket for object to retrieve
    :param key: The key of the object to be retrieved from the bucket
    :param uri: An s3:// path containing the bucket and key of the object
    :param use_threads: Enables the use_threads transfer config
    :param s3_resource: Boto3 resource to use if you don't wish to use the default resource
    :return: Path of the local file
    """
    config = TransferConfig(use_threads=use_threads)
    s3_object_local = os.path.join(directory, key.split('/')[-1])
    try:
        s3_resource.Bucket(bucket).download_file(key, s3_object_local, Config=config)
    except botocore.exceptions.ClientError as e:
        if e.response['Error']['Code'] == "404":
            print("The object does not exist.")
        else:
            raise
    return s3_object_local


@__load_resource
@__decompose_location(require_key=True)
def download_to_temp(*location, bucket=None, key=None, uri=None, s3_resource=None):
    """
    Downloads the an S3 object to a temp directory on the local file system.
    :param bucket: The S3 bucket for object to retrieve
    :param key: The key of the object to be retrieved from the bucket
    :param uri: An s3:// path containing the bucket and key of the object
    :param s3_resource: Boto3 resource to use if you don't wish to use the default resource
    :return: Path of the local file
    """
    temp_dir = _create_temp_dir()
    file = os.path.join(temp_dir, key.split('/')[-1])
    if not os.path.isfile(file):
        print('starting download')
        download(temp_dir, bucket, key, use_threads=True, s3_resource=s3_resource)
        print('download complete')
    return file


def _create_temp_dir():
    """
    Creates a temp directory in the current path.
    :return: The path of the temp directory
    """
    _temp_dir = os.getcwd() + "/temp"
    if not os.path.isdir(_temp_dir):
        os.makedirs(_temp_dir)
    return _temp_dir


@__load_resource
@__decompose_location(require_key=True)
def upload(file_name, *location, bucket=None, key=None, uri=None, s3_resource=None):
    s3_resource.Bucket(bucket).upload_file(file_name, key)
    return compose_uri(bucket, key)


@__load_resource
def write_temp_object(value, prefix, acl=None, s3_resource=None, bucket_identifier=None, region=None,
                      bucket=None):
    """
    Write an object to a temp bucket with a unique UUID.
    :param value: Object to write to S3
    :param prefix: Prefix to attach ahead of the UUID as the key
    :param acl: The canned ACL to apply to the object
    :param s3_resource: Boto3 resource to use if you don't wish to use the default resource
    :param bucket_identifier: The identifier to attach to the temp bucket that will be used for writing to s3, typically
    the account id (from STS) for the account being used
    :param region: The s3 region to store the data in
    :param bucket: The bucket to use instead of creating/using a temp bucket
    :return: The URI of the object written to S3
    """
    s3_resource = s3_resource if s3_resource else resource
    if bucket is None:
        bucket = get_temp_bucket(region=region, bucket_identifier=bucket_identifier, s3_resource=s3_resource)
    key = prefix + str(uuid.uuid4())
    return write_object(value, bucket=bucket, key=key, acl=acl, s3_resource=s3_resource)


@__load_resource
@__decompose_location(require_key=True)
def make_public(*location, bucket=None, key=None, uri=None, s3_resource=None):
    """
    Makes the object defined by the bucket/key pair (or uri) public.
    :param bucket: The S3 bucket for object
    :param key: The key of the object
    :param uri: An s3:// path containing the bucket and key of the object
    :param s3_resource: Boto3 resource to use if you don't wish to use the default resource
    :return: The URL of the object
    """
    s3_resource.meta.client.put_object_acl(Bucket=bucket, Key=key, ACL='public-read')
    return get_public_url(bucket=bucket, key=key)


@__decompose_location()
def get_public_url(*location, bucket=None, key=None, uri=None):
    """
    Returns the public URL of an S3 object (assuming it's public).
    :param bucket: The S3 bucket for object
    :param key: The key of the object
    :param uri: An s3:// path containing the bucket and key of the object
    :return: The URL of the object
    """
    if key:
        return 'https://{}.s3.amazonaws.com/{}'.format(bucket, parse.quote(key))
    else:
        return 'https://{}.s3.amazonaws.com'.format(bucket)


@__load_resource
def create_bucket(bucket, acl='private', region=__session.region_name, s3_resource=None):
    bucket_obj = s3_resource.Bucket(bucket)
    bucket_obj.load()
    if bucket_obj.creation_date is None:
        bucket_obj.create(ACL=acl, CreateBucketConfiguration={'LocationConstraint': region})
        bucket_obj.wait_until_exists()
    return bucket_obj


@__load_resource
def delete_bucket(bucket, s3_resource=None):
    bucket_obj = s3_resource.Bucket(bucket)
    bucket_obj.delete()


@__load_resource
def get_temp_bucket(region=None, s3_resource=None, bucket_identifier=None):
    """
    Create a bucket that will be used as temp storage for larry commands.
    The bucket will be created in the region associated with the current session
    using a name based on the current session account id and region.
    :param region: Region to locate the temp bucket
    :param s3_resource: Boto3 resource to use if you don't wish to use the default resource
    :param bucket_identifier: The bucket identifier to use as a unique identifier for the bucket, defaults to the
    account id associated with the session
    :return: The name of the created bucket
    """
    if region is None:
        region = __session.region_name
    if bucket_identifier is None:
        bucket_identifier = sts.account_id()
    bucket = '{}-larry-{}'.format(bucket_identifier, region)
    create_bucket(bucket, region=region, s3_resource=s3_resource)
    return bucket


# TODO: add filter parameter
# TODO: rationalize the list params
def download_to_zip(file, bucket, prefix=None, prefixes=None):
    if prefix:
        prefixes = [prefix]
    with ZipFile(file, 'w') as zfile:
        for prefix in prefixes:
            for key in list_objects(bucket, prefix):
                zfile.writestr(parse.quote(key), data=read(bucket, key))


def file_name_portion(uri):
    file = decompose_uri(uri)[1].split('/')[-1]
    return file[:file.rfind('.')]



from larry import utils
import boto3
import json

# Local IAM resource object
resource = None
# A local instance of the boto3 session to use
__session = boto3.session.Session()


def __assume_role_service_policy(service):
    return json.dumps({
        "Version": "2012-10-17",
        "Statement": [
            {
                "Effect": "Allow",
                "Principal": {
                    "Service": "{}.amazonaws.com".format(service)
                },
                "Action": "sts:AssumeRole"}
        ]
    })


def set_session(aws_access_key_id=None,
                aws_secret_access_key=None,
                aws__session_token=None,
                region_name=None,
                profile_name=None,
                boto_session=None):
    """
    Sets the boto3 session for this module to use a specified configuration state.
    :param aws_access_key_id: AWS access key ID
    :param aws_secret_access_key: AWS secret access key
    :param aws__session_token: AWS temporary session token
    :param region_name: Default region when creating new connections
    :param profile_name: The name of a profile to use
    :param boto_session: An existing session to use
    :return: None
    """
    global __session, resource
    __session = boto_session if boto_session is not None else boto3.session.Session(**utils.copy_non_null_keys(locals()))
    resource = __session.resource('iam')


def create_service_role(name, service, policy=None):
    role = resource.create_role(RoleName=name,
                                AssumeRolePolicyDocument=__assume_role_service_policy(service))
    if policy:
        if isinstance(policy, list):
            for p in policy:
                role.attach_policy(PolicyArn=p)
        else:
            role.attach_policy(PolicyArn=policy)
    return role.arn


class policies():
    AWSDirectConnectReadOnlyAccess = 'arn:aws:iam::aws:policy/AWSDirectConnectReadOnlyAccess'
    AmazonGlacierReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonGlacierReadOnlyAccess'
    AWSMarketplaceFullAccess = 'arn:aws:iam::aws:policy/AWSMarketplaceFullAccess'
    ClientVPNServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/ClientVPNServiceRolePolicy'
    AWSSSODirectoryAdministrator = 'arn:aws:iam::aws:policy/AWSSSODirectoryAdministrator'
    AWSIoT1ClickReadOnlyAccess = 'arn:aws:iam::aws:policy/AWSIoT1ClickReadOnlyAccess'
    AutoScalingConsoleReadOnlyAccess = 'arn:aws:iam::aws:policy/AutoScalingConsoleReadOnlyAccess'
    AmazonDMSRedshiftS3Role = 'arn:aws:iam::aws:policy/service-role/AmazonDMSRedshiftS3Role'
    AWSQuickSightListIAM = 'arn:aws:iam::aws:policy/service-role/AWSQuickSightListIAM'
    AWSHealthFullAccess = 'arn:aws:iam::aws:policy/AWSHealthFullAccess'
    AlexaForBusinessGatewayExecution = 'arn:aws:iam::aws:policy/AlexaForBusinessGatewayExecution'
    AmazonElasticTranscoder_ReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonElasticTranscoder_ReadOnlyAccess'
    AmazonRDSFullAccess = 'arn:aws:iam::aws:policy/AmazonRDSFullAccess'
    SupportUser = 'arn:aws:iam::aws:policy/job-function/SupportUser'
    AmazonEC2FullAccess = 'arn:aws:iam::aws:policy/AmazonEC2FullAccess'
    SecretsManagerReadWrite = 'arn:aws:iam::aws:policy/SecretsManagerReadWrite'
    AWSIoTThingsRegistration = 'arn:aws:iam::aws:policy/service-role/AWSIoTThingsRegistration'
    AmazonDocDBReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonDocDBReadOnlyAccess'
    AWSElasticBeanstalkReadOnlyAccess = 'arn:aws:iam::aws:policy/AWSElasticBeanstalkReadOnlyAccess'
    AmazonMQApiFullAccess = 'arn:aws:iam::aws:policy/AmazonMQApiFullAccess'
    AWSElementalMediaStoreReadOnly = 'arn:aws:iam::aws:policy/AWSElementalMediaStoreReadOnly'
    AWSCertificateManagerReadOnly = 'arn:aws:iam::aws:policy/AWSCertificateManagerReadOnly'
    AWSQuicksightAthenaAccess = 'arn:aws:iam::aws:policy/service-role/AWSQuicksightAthenaAccess'
    AWSCloudMapRegisterInstanceAccess = 'arn:aws:iam::aws:policy/AWSCloudMapRegisterInstanceAccess'
    AWSMarketplaceImageBuildFullAccess = 'arn:aws:iam::aws:policy/AWSMarketplaceImageBuildFullAccess'
    AWSCodeCommitPowerUser = 'arn:aws:iam::aws:policy/AWSCodeCommitPowerUser'
    AWSCodeCommitFullAccess = 'arn:aws:iam::aws:policy/AWSCodeCommitFullAccess'
    IAMSelfManageServiceSpecificCredentials = 'arn:aws:iam::aws:policy/IAMSelfManageServiceSpecificCredentials'
    AmazonEMRCleanupPolicy = 'arn:aws:iam::aws:policy/aws-service-role/AmazonEMRCleanupPolicy'
    AWSCloud9EnvironmentMember = 'arn:aws:iam::aws:policy/AWSCloud9EnvironmentMember'
    AWSApplicationAutoscalingSageMakerEndpointPolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSApplicationAutoscalingSageMakerEndpointPolicy'
    FMSServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/FMSServiceRolePolicy'
    AmazonSQSFullAccess = 'arn:aws:iam::aws:policy/AmazonSQSFullAccess'
    AlexaForBusinessReadOnlyAccess = 'arn:aws:iam::aws:policy/AlexaForBusinessReadOnlyAccess'
    AWSLambdaFullAccess = 'arn:aws:iam::aws:policy/AWSLambdaFullAccess'
    AWSIoTLogging = 'arn:aws:iam::aws:policy/service-role/AWSIoTLogging'
    AmazonEC2RoleforSSM = 'arn:aws:iam::aws:policy/service-role/AmazonEC2RoleforSSM'
    AlexaForBusinessNetworkProfileServicePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AlexaForBusinessNetworkProfileServicePolicy'
    AWSCloudHSMRole = 'arn:aws:iam::aws:policy/service-role/AWSCloudHSMRole'
    AWSEnhancedClassicNetworkingMangementPolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSEnhancedClassicNetworkingMangementPolicy'
    IAMFullAccess = 'arn:aws:iam::aws:policy/IAMFullAccess'
    AmazonInspectorFullAccess = 'arn:aws:iam::aws:policy/AmazonInspectorFullAccess'
    AmazonElastiCacheFullAccess = 'arn:aws:iam::aws:policy/AmazonElastiCacheFullAccess'
    AWSAgentlessDiscoveryService = 'arn:aws:iam::aws:policy/AWSAgentlessDiscoveryService'
    AWSXrayWriteOnlyAccess = 'arn:aws:iam::aws:policy/AWSXrayWriteOnlyAccess'
    AWSPriceListServiceFullAccess = 'arn:aws:iam::aws:policy/AWSPriceListServiceFullAccess'
    AWSKeyManagementServiceCustomKeyStoresServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSKeyManagementServiceCustomKeyStoresServiceRolePolicy'
    AutoScalingReadOnlyAccess = 'arn:aws:iam::aws:policy/AutoScalingReadOnlyAccess'
    AmazonForecastFullAccess = 'arn:aws:iam::aws:policy/AmazonForecastFullAccess'
    AmazonWorkLinkReadOnly = 'arn:aws:iam::aws:policy/AmazonWorkLinkReadOnly'
    TranslateFullAccess = 'arn:aws:iam::aws:policy/TranslateFullAccess'
    AutoScalingFullAccess = 'arn:aws:iam::aws:policy/AutoScalingFullAccess'
    AmazonEC2RoleforAWSCodeDeploy = 'arn:aws:iam::aws:policy/service-role/AmazonEC2RoleforAWSCodeDeploy'
    AWSFMMemberReadOnlyAccess = 'arn:aws:iam::aws:policy/AWSFMMemberReadOnlyAccess'
    AmazonElasticMapReduceEditorsRole = 'arn:aws:iam::aws:policy/service-role/AmazonElasticMapReduceEditorsRole'
    AmazonEKSClusterPolicy = 'arn:aws:iam::aws:policy/AmazonEKSClusterPolicy'
    AmazonEKSWorkerNodePolicy = 'arn:aws:iam::aws:policy/AmazonEKSWorkerNodePolicy'
    AWSMobileHub_ReadOnly = 'arn:aws:iam::aws:policy/AWSMobileHub_ReadOnly'
    CloudWatchEventsBuiltInTargetExecutionAccess = 'arn:aws:iam::aws:policy/service-role/CloudWatchEventsBuiltInTargetExecutionAccess'
    AutoScalingServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AutoScalingServiceRolePolicy'
    AmazonElasticTranscoder_FullAccess = 'arn:aws:iam::aws:policy/AmazonElasticTranscoder_FullAccess'
    AmazonCloudDirectoryReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonCloudDirectoryReadOnlyAccess'
    CloudWatchAgentAdminPolicy = 'arn:aws:iam::aws:policy/CloudWatchAgentAdminPolicy'
    AWSOpsWorksFullAccess = 'arn:aws:iam::aws:policy/AWSOpsWorksFullAccess'
    AWSOpsWorksCMInstanceProfileRole = 'arn:aws:iam::aws:policy/AWSOpsWorksCMInstanceProfileRole'
    AWSBatchServiceEventTargetRole = 'arn:aws:iam::aws:policy/service-role/AWSBatchServiceEventTargetRole'
    AWSCodePipelineApproverAccess = 'arn:aws:iam::aws:policy/AWSCodePipelineApproverAccess'
    AWSApplicationDiscoveryAgentAccess = 'arn:aws:iam::aws:policy/AWSApplicationDiscoveryAgentAccess'
    ViewOnlyAccess = 'arn:aws:iam::aws:policy/job-function/ViewOnlyAccess'
    AmazonElasticMapReduceRole = 'arn:aws:iam::aws:policy/service-role/AmazonElasticMapReduceRole'
    ElasticLoadBalancingFullAccess = 'arn:aws:iam::aws:policy/ElasticLoadBalancingFullAccess'
    AmazonRoute53DomainsReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonRoute53DomainsReadOnlyAccess'
    AmazonSSMAutomationApproverAccess = 'arn:aws:iam::aws:policy/AmazonSSMAutomationApproverAccess'
    AWSOpsWorksRole = 'arn:aws:iam::aws:policy/service-role/AWSOpsWorksRole'
    AWSSecurityHubReadOnlyAccess = 'arn:aws:iam::aws:policy/AWSSecurityHubReadOnlyAccess'
    AWSConfigRoleForOrganizations = 'arn:aws:iam::aws:policy/service-role/AWSConfigRoleForOrganizations'
    ApplicationAutoScalingForAmazonAppStreamAccess = 'arn:aws:iam::aws:policy/service-role/ApplicationAutoScalingForAmazonAppStreamAccess'
    AmazonEC2ContainerRegistryFullAccess = 'arn:aws:iam::aws:policy/AmazonEC2ContainerRegistryFullAccess'
    AmazonFSxFullAccess = 'arn:aws:iam::aws:policy/AmazonFSxFullAccess'
    SimpleWorkflowFullAccess = 'arn:aws:iam::aws:policy/SimpleWorkflowFullAccess'
    GreengrassOTAUpdateArtifactAccess = 'arn:aws:iam::aws:policy/service-role/GreengrassOTAUpdateArtifactAccess'
    AmazonS3FullAccess = 'arn:aws:iam::aws:policy/AmazonS3FullAccess'
    AWSStorageGatewayReadOnlyAccess = 'arn:aws:iam::aws:policy/AWSStorageGatewayReadOnlyAccess'
    Billing = 'arn:aws:iam::aws:policy/job-function/Billing'
    QuickSightAccessForS3StorageManagementAnalyticsReadOnly = 'arn:aws:iam::aws:policy/service-role/QuickSightAccessForS3StorageManagementAnalyticsReadOnly'
    AmazonEC2ContainerRegistryReadOnly = 'arn:aws:iam::aws:policy/AmazonEC2ContainerRegistryReadOnly'
    AWSRoboMakerFullAccess = 'arn:aws:iam::aws:policy/AWSRoboMakerFullAccess'
    AmazonElasticMapReduceforEC2Role = 'arn:aws:iam::aws:policy/service-role/AmazonElasticMapReduceforEC2Role'
    DatabaseAdministrator = 'arn:aws:iam::aws:policy/job-function/DatabaseAdministrator'
    AmazonRedshiftReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonRedshiftReadOnlyAccess'
    AmazonEC2ReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonEC2ReadOnlyAccess'
    CloudWatchAgentServerPolicy = 'arn:aws:iam::aws:policy/CloudWatchAgentServerPolicy'
    AWSXrayReadOnlyAccess = 'arn:aws:iam::aws:policy/AWSXrayReadOnlyAccess'
    AWSElasticBeanstalkEnhancedHealth = 'arn:aws:iam::aws:policy/service-role/AWSElasticBeanstalkEnhancedHealth'
    WellArchitectedConsoleFullAccess = 'arn:aws:iam::aws:policy/WellArchitectedConsoleFullAccess'
    AmazonElasticMapReduceReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonElasticMapReduceReadOnlyAccess'
    AWSDirectoryServiceReadOnlyAccess = 'arn:aws:iam::aws:policy/AWSDirectoryServiceReadOnlyAccess'
    AWSSSOMasterAccountAdministrator = 'arn:aws:iam::aws:policy/AWSSSOMasterAccountAdministrator'
    AmazonGuardDutyServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AmazonGuardDutyServiceRolePolicy'
    AmazonVPCReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonVPCReadOnlyAccess'
    AWSElasticBeanstalkServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSElasticBeanstalkServiceRolePolicy'
    ServerMigrationServiceLaunchRole = 'arn:aws:iam::aws:policy/service-role/ServerMigrationServiceLaunchRole'
    AWSCodeDeployRoleForECS = 'arn:aws:iam::aws:policy/AWSCodeDeployRoleForECS'
    CloudWatchEventsReadOnlyAccess = 'arn:aws:iam::aws:policy/CloudWatchEventsReadOnlyAccess'
    AWSLambdaReplicator = 'arn:aws:iam::aws:policy/aws-service-role/AWSLambdaReplicator'
    AmazonAPIGatewayInvokeFullAccess = 'arn:aws:iam::aws:policy/AmazonAPIGatewayInvokeFullAccess'
    AWSSSOServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSSSOServiceRolePolicy'
    AWSLicenseManagerMasterAccountRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSLicenseManagerMasterAccountRolePolicy'
    AmazonKinesisAnalyticsReadOnly = 'arn:aws:iam::aws:policy/AmazonKinesisAnalyticsReadOnly'
    AmazonMobileAnalyticsFullAccess = 'arn:aws:iam::aws:policy/AmazonMobileAnalyticsFullAccess'
    AWSMobileHub_FullAccess = 'arn:aws:iam::aws:policy/AWSMobileHub_FullAccess'
    AmazonAPIGatewayPushToCloudWatchLogs = 'arn:aws:iam::aws:policy/service-role/AmazonAPIGatewayPushToCloudWatchLogs'
    AWSDataPipelineRole = 'arn:aws:iam::aws:policy/service-role/AWSDataPipelineRole'
    CloudWatchFullAccess = 'arn:aws:iam::aws:policy/CloudWatchFullAccess'
    AmazonMQApiReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonMQApiReadOnlyAccess'
    AWSDeepLensLambdaFunctionAccessPolicy = 'arn:aws:iam::aws:policy/AWSDeepLensLambdaFunctionAccessPolicy'
    AmazonGuardDutyFullAccess = 'arn:aws:iam::aws:policy/AmazonGuardDutyFullAccess'
    AmazonRDSDirectoryServiceAccess = 'arn:aws:iam::aws:policy/service-role/AmazonRDSDirectoryServiceAccess'
    AWSCodePipelineReadOnlyAccess = 'arn:aws:iam::aws:policy/AWSCodePipelineReadOnlyAccess'
    ReadOnlyAccess = 'arn:aws:iam::aws:policy/ReadOnlyAccess'
    AWSAppSyncInvokeFullAccess = 'arn:aws:iam::aws:policy/AWSAppSyncInvokeFullAccess'
    AmazonMachineLearningBatchPredictionsAccess = 'arn:aws:iam::aws:policy/AmazonMachineLearningBatchPredictionsAccess'
    AWSIoTSiteWiseFullAccess = 'arn:aws:iam::aws:policy/AWSIoTSiteWiseFullAccess'
    AlexaForBusinessFullAccess = 'arn:aws:iam::aws:policy/AlexaForBusinessFullAccess'
    AWSEC2SpotFleetServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSEC2SpotFleetServiceRolePolicy'
    AmazonRekognitionReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonRekognitionReadOnlyAccess'
    AWSCodeDeployReadOnlyAccess = 'arn:aws:iam::aws:policy/AWSCodeDeployReadOnlyAccess'
    CloudSearchFullAccess = 'arn:aws:iam::aws:policy/CloudSearchFullAccess'
    AWSLicenseManagerServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSLicenseManagerServiceRolePolicy'
    AWSCloudHSMFullAccess = 'arn:aws:iam::aws:policy/AWSCloudHSMFullAccess'
    AmazonEC2SpotFleetAutoscaleRole = 'arn:aws:iam::aws:policy/service-role/AmazonEC2SpotFleetAutoscaleRole'
    AWSElasticLoadBalancingServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSElasticLoadBalancingServiceRolePolicy'
    AWSCodeBuildDeveloperAccess = 'arn:aws:iam::aws:policy/AWSCodeBuildDeveloperAccess'
    ElastiCacheServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/ElastiCacheServiceRolePolicy'
    AWSGlueServiceNotebookRole = 'arn:aws:iam::aws:policy/service-role/AWSGlueServiceNotebookRole'
    AWSDataPipeline_PowerUser = 'arn:aws:iam::aws:policy/AWSDataPipeline_PowerUser'
    AWSCodeStarServiceRole = 'arn:aws:iam::aws:policy/service-role/AWSCodeStarServiceRole'
    AmazonTranscribeFullAccess = 'arn:aws:iam::aws:policy/AmazonTranscribeFullAccess'
    AWSDirectoryServiceFullAccess = 'arn:aws:iam::aws:policy/AWSDirectoryServiceFullAccess'
    AmazonFreeRTOSOTAUpdate = 'arn:aws:iam::aws:policy/service-role/AmazonFreeRTOSOTAUpdate'
    AmazonWorkLinkServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AmazonWorkLinkServiceRolePolicy'
    AmazonDynamoDBFullAccess = 'arn:aws:iam::aws:policy/AmazonDynamoDBFullAccess'
    AmazonSESReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonSESReadOnlyAccess'
    AmazonRedshiftQueryEditor = 'arn:aws:iam::aws:policy/AmazonRedshiftQueryEditor'
    AWSWAFReadOnlyAccess = 'arn:aws:iam::aws:policy/AWSWAFReadOnlyAccess'
    AutoScalingNotificationAccessRole = 'arn:aws:iam::aws:policy/service-role/AutoScalingNotificationAccessRole'
    AmazonMechanicalTurkReadOnly = 'arn:aws:iam::aws:policy/AmazonMechanicalTurkReadOnly'
    AmazonKinesisReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonKinesisReadOnlyAccess'
    AWSXRayDaemonWriteAccess = 'arn:aws:iam::aws:policy/AWSXRayDaemonWriteAccess'
    AWSCloudMapReadOnlyAccess = 'arn:aws:iam::aws:policy/AWSCloudMapReadOnlyAccess'
    AWSCloudFrontLogger = 'arn:aws:iam::aws:policy/aws-service-role/AWSCloudFrontLogger'
    AWSCodeDeployFullAccess = 'arn:aws:iam::aws:policy/AWSCodeDeployFullAccess'
    AWSBackupServiceRolePolicyForBackup = 'arn:aws:iam::aws:policy/service-role/AWSBackupServiceRolePolicyForBackup'
    AWSRoboMakerServiceRolePolicy = 'arn:aws:iam::aws:policy/AWSRoboMakerServiceRolePolicy'
    CloudWatchActionsEC2Access = 'arn:aws:iam::aws:policy/CloudWatchActionsEC2Access'
    AWSLambdaDynamoDBExecutionRole = 'arn:aws:iam::aws:policy/service-role/AWSLambdaDynamoDBExecutionRole'
    AmazonRoute53DomainsFullAccess = 'arn:aws:iam::aws:policy/AmazonRoute53DomainsFullAccess'
    AmazonElastiCacheReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonElastiCacheReadOnlyAccess'
    AmazonRDSServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AmazonRDSServiceRolePolicy'
    AmazonAthenaFullAccess = 'arn:aws:iam::aws:policy/AmazonAthenaFullAccess'
    AmazonElasticFileSystemReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonElasticFileSystemReadOnlyAccess'
    AWSCloudMapDiscoverInstanceAccess = 'arn:aws:iam::aws:policy/AWSCloudMapDiscoverInstanceAccess'
    CloudFrontFullAccess = 'arn:aws:iam::aws:policy/CloudFrontFullAccess'
    AmazonConnectFullAccess = 'arn:aws:iam::aws:policy/AmazonConnectFullAccess'
    AWSCloud9Administrator = 'arn:aws:iam::aws:policy/AWSCloud9Administrator'
    AWSApplicationAutoscalingEMRInstanceGroupPolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSApplicationAutoscalingEMRInstanceGroupPolicy'
    AmazonTextractFullAccess = 'arn:aws:iam::aws:policy/AmazonTextractFullAccess'
    AWSOrganizationsServiceTrustPolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSOrganizationsServiceTrustPolicy'
    AmazonDocDBFullAccess = 'arn:aws:iam::aws:policy/AmazonDocDBFullAccess'
    AmazonMobileAnalyticsNonfinancialReportAccess = 'arn:aws:iam::aws:policy/AmazonMobileAnalyticsNon-financialReportAccess'
    AWSCloudTrailFullAccess = 'arn:aws:iam::aws:policy/AWSCloudTrailFullAccess'
    AmazonCognitoDeveloperAuthenticatedIdentities = 'arn:aws:iam::aws:policy/AmazonCognitoDeveloperAuthenticatedIdentities'
    AWSConfigRole = 'arn:aws:iam::aws:policy/service-role/AWSConfigRole'
    AWSSSOMemberAccountAdministrator = 'arn:aws:iam::aws:policy/AWSSSOMemberAccountAdministrator'
    AWSApplicationAutoscalingAppStreamFleetPolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSApplicationAutoscalingAppStreamFleetPolicy'
    AWSCertificateManagerPrivateCAFullAccess = 'arn:aws:iam::aws:policy/AWSCertificateManagerPrivateCAFullAccess'
    AWSGlueServiceRole = 'arn:aws:iam::aws:policy/service-role/AWSGlueServiceRole'
    AmazonAppStreamServiceAccess = 'arn:aws:iam::aws:policy/service-role/AmazonAppStreamServiceAccess'
    AmazonRedshiftFullAccess = 'arn:aws:iam::aws:policy/AmazonRedshiftFullAccess'
    AWSTransferLoggingAccess = 'arn:aws:iam::aws:policy/service-role/AWSTransferLoggingAccess'
    AmazonZocaloReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonZocaloReadOnlyAccess'
    AWSCloudHSMReadOnlyAccess = 'arn:aws:iam::aws:policy/AWSCloudHSMReadOnlyAccess'
    ComprehendFullAccess = 'arn:aws:iam::aws:policy/ComprehendFullAccess'
    AmazonFSxConsoleFullAccess = 'arn:aws:iam::aws:policy/AmazonFSxConsoleFullAccess'
    SystemAdministrator = 'arn:aws:iam::aws:policy/job-function/SystemAdministrator'
    AmazonEC2ContainerServiceEventsRole = 'arn:aws:iam::aws:policy/service-role/AmazonEC2ContainerServiceEventsRole'
    AmazonRoute53ReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonRoute53ReadOnlyAccess'
    AWSMigrationHubDiscoveryAccess = 'arn:aws:iam::aws:policy/service-role/AWSMigrationHubDiscoveryAccess'
    AmazonEC2ContainerServiceAutoscaleRole = 'arn:aws:iam::aws:policy/service-role/AmazonEC2ContainerServiceAutoscaleRole'
    AWSAppSyncSchemaAuthor = 'arn:aws:iam::aws:policy/AWSAppSyncSchemaAuthor'
    AlexaForBusinessDeviceSetup = 'arn:aws:iam::aws:policy/AlexaForBusinessDeviceSetup'
    AWSBatchServiceRole = 'arn:aws:iam::aws:policy/service-role/AWSBatchServiceRole'
    AWSElasticBeanstalkWebTier = 'arn:aws:iam::aws:policy/AWSElasticBeanstalkWebTier'
    AmazonSQSReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonSQSReadOnlyAccess'
    AmazonChimeFullAccess = 'arn:aws:iam::aws:policy/AmazonChimeFullAccess'
    AWSDeepRacerRoboMakerAccessPolicy = 'arn:aws:iam::aws:policy/AWSDeepRacerRoboMakerAccessPolicy'
    AWSElasticLoadBalancingClassicServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSElasticLoadBalancingClassicServiceRolePolicy'
    AWSMigrationHubDMSAccess = 'arn:aws:iam::aws:policy/service-role/AWSMigrationHubDMSAccess'
    WellArchitectedConsoleReadOnlyAccess = 'arn:aws:iam::aws:policy/WellArchitectedConsoleReadOnlyAccess'
    AmazonKinesisFullAccess = 'arn:aws:iam::aws:policy/AmazonKinesisFullAccess'
    AmazonGuardDutyReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonGuardDutyReadOnlyAccess'
    AmazonFSxServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AmazonFSxServiceRolePolicy'
    AmazonECSServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AmazonECSServiceRolePolicy'
    AmazonConnectReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonConnectReadOnlyAccess'
    AmazonMachineLearningReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonMachineLearningReadOnlyAccess'
    AmazonRekognitionFullAccess = 'arn:aws:iam::aws:policy/AmazonRekognitionFullAccess'
    RDSCloudHsmAuthorizationRole = 'arn:aws:iam::aws:policy/service-role/RDSCloudHsmAuthorizationRole'
    AmazonMachineLearningFullAccess = 'arn:aws:iam::aws:policy/AmazonMachineLearningFullAccess'
    AdministratorAccess = 'arn:aws:iam::aws:policy/AdministratorAccess'
    AmazonMachineLearningRealTimePredictionOnlyAccess = 'arn:aws:iam::aws:policy/AmazonMachineLearningRealTimePredictionOnlyAccess'
    AWSAppSyncPushToCloudWatchLogs = 'arn:aws:iam::aws:policy/service-role/AWSAppSyncPushToCloudWatchLogs'
    AWSMigrationHubSMSAccess = 'arn:aws:iam::aws:policy/service-role/AWSMigrationHubSMSAccess'
    AWSB9InternalServicePolicy = 'arn:aws:iam::aws:policy/AWSB9InternalServicePolicy'
    AWSConfigUserAccess = 'arn:aws:iam::aws:policy/AWSConfigUserAccess'
    AWSIoTConfigAccess = 'arn:aws:iam::aws:policy/AWSIoTConfigAccess'
    SecurityAudit = 'arn:aws:iam::aws:policy/SecurityAudit'
    AWSDiscoveryContinuousExportFirehosePolicy = 'arn:aws:iam::aws:policy/AWSDiscoveryContinuousExportFirehosePolicy'
    AmazonCognitoIdpEmailServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AmazonCognitoIdpEmailServiceRolePolicy'
    AWSElementalMediaConvertFullAccess = 'arn:aws:iam::aws:policy/AWSElementalMediaConvertFullAccess'
    AWSRoboMakerReadOnlyAccess = 'arn:aws:iam::aws:policy/AWSRoboMakerReadOnlyAccess'
    AWSResourceGroupsReadOnlyAccess = 'arn:aws:iam::aws:policy/AWSResourceGroupsReadOnlyAccess'
    AWSCodeStarFullAccess = 'arn:aws:iam::aws:policy/AWSCodeStarFullAccess'
    AmazonSSMServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AmazonSSMServiceRolePolicy'
    AWSDataPipeline_FullAccess = 'arn:aws:iam::aws:policy/AWSDataPipeline_FullAccess'
    NeptuneFullAccess = 'arn:aws:iam::aws:policy/NeptuneFullAccess'
    AmazonSSMManagedInstanceCore = 'arn:aws:iam::aws:policy/AmazonSSMManagedInstanceCore'
    AWSAutoScalingPlansEC2AutoScalingPolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSAutoScalingPlansEC2AutoScalingPolicy'
    AmazonDynamoDBReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonDynamoDBReadOnlyAccess'
    AutoScalingConsoleFullAccess = 'arn:aws:iam::aws:policy/AutoScalingConsoleFullAccess'
    AWSElementalMediaPackageFullAccess = 'arn:aws:iam::aws:policy/AWSElementalMediaPackageFullAccess'
    AmazonKinesisVideoStreamsFullAccess = 'arn:aws:iam::aws:policy/AmazonKinesisVideoStreamsFullAccess'
    AmazonSNSReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonSNSReadOnlyAccess'
    AmazonRDSPreviewServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AmazonRDSPreviewServiceRolePolicy'
    AWSEC2SpotServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSEC2SpotServiceRolePolicy'
    AmazonElasticMapReduceFullAccess = 'arn:aws:iam::aws:policy/AmazonElasticMapReduceFullAccess'
    AWSCloudMapFullAccess = 'arn:aws:iam::aws:policy/AWSCloudMapFullAccess'
    AWSDataLifecycleManagerServiceRole = 'arn:aws:iam::aws:policy/service-role/AWSDataLifecycleManagerServiceRole'
    AmazonS3ReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonS3ReadOnlyAccess'
    AWSElasticBeanstalkFullAccess = 'arn:aws:iam::aws:policy/AWSElasticBeanstalkFullAccess'
    AmazonWorkSpacesAdmin = 'arn:aws:iam::aws:policy/AmazonWorkSpacesAdmin'
    AWSCodeDeployRole = 'arn:aws:iam::aws:policy/service-role/AWSCodeDeployRole'
    AmazonSESFullAccess = 'arn:aws:iam::aws:policy/AmazonSESFullAccess'
    CloudWatchLogsReadOnlyAccess = 'arn:aws:iam::aws:policy/CloudWatchLogsReadOnlyAccess'
    AmazonRDSBetaServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AmazonRDSBetaServiceRolePolicy'
    AmazonKinesisFirehoseReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonKinesisFirehoseReadOnlyAccess'
    GlobalAcceleratorFullAccess = 'arn:aws:iam::aws:policy/GlobalAcceleratorFullAccess'
    AmazonDynamoDBFullAccesswithDataPipeline = 'arn:aws:iam::aws:policy/AmazonDynamoDBFullAccesswithDataPipeline'
    AWSIoTAnalyticsReadOnlyAccess = 'arn:aws:iam::aws:policy/AWSIoTAnalyticsReadOnlyAccess'
    AmazonEC2RoleforDataPipelineRole = 'arn:aws:iam::aws:policy/service-role/AmazonEC2RoleforDataPipelineRole'
    CloudWatchLogsFullAccess = 'arn:aws:iam::aws:policy/CloudWatchLogsFullAccess'
    AWSSecurityHubFullAccess = 'arn:aws:iam::aws:policy/AWSSecurityHubFullAccess'
    AWSElementalMediaPackageReadOnly = 'arn:aws:iam::aws:policy/AWSElementalMediaPackageReadOnly'
    AWSElasticBeanstalkMulticontainerDocker = 'arn:aws:iam::aws:policy/AWSElasticBeanstalkMulticontainerDocker'
    AmazonPersonalizeFullAccess = 'arn:aws:iam::aws:policy/service-role/AmazonPersonalizeFullAccess'
    AWSMigrationHubFullAccess = 'arn:aws:iam::aws:policy/AWSMigrationHubFullAccess'
    AmazonFSxReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonFSxReadOnlyAccess'
    IAMUserChangePassword = 'arn:aws:iam::aws:policy/IAMUserChangePassword'
    LightsailExportAccess = 'arn:aws:iam::aws:policy/aws-service-role/LightsailExportAccess'
    AmazonAPIGatewayAdministrator = 'arn:aws:iam::aws:policy/AmazonAPIGatewayAdministrator'
    AmazonVPCCrossAccountNetworkInterfaceOperations = 'arn:aws:iam::aws:policy/AmazonVPCCrossAccountNetworkInterfaceOperations'
    AmazonMacieSetupRole = 'arn:aws:iam::aws:policy/service-role/AmazonMacieSetupRole'
    AmazonPollyReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonPollyReadOnlyAccess'
    AmazonRDSDataFullAccess = 'arn:aws:iam::aws:policy/AmazonRDSDataFullAccess'
    AmazonMobileAnalyticsWriteOnlyAccess = 'arn:aws:iam::aws:policy/AmazonMobileAnalyticsWriteOnlyAccess'
    AmazonEC2SpotFleetTaggingRole = 'arn:aws:iam::aws:policy/service-role/AmazonEC2SpotFleetTaggingRole'
    DataScientist = 'arn:aws:iam::aws:policy/job-function/DataScientist'
    AWSMarketplaceMeteringFullAccess = 'arn:aws:iam::aws:policy/AWSMarketplaceMeteringFullAccess'
    AWSOpsWorksCMServiceRole = 'arn:aws:iam::aws:policy/service-role/AWSOpsWorksCMServiceRole'
    FSxDeleteServiceLinkedRoleAccess = 'arn:aws:iam::aws:policy/aws-service-role/FSxDeleteServiceLinkedRoleAccess'
    WorkLinkServiceRolePolicy = 'arn:aws:iam::aws:policy/WorkLinkServiceRolePolicy'
    AmazonConnectServiceLinkedRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AmazonConnectServiceLinkedRolePolicy'
    AWSPrivateMarketplaceAdminFullAccess = 'arn:aws:iam::aws:policy/AWSPrivateMarketplaceAdminFullAccess'
    AWSConnector = 'arn:aws:iam::aws:policy/AWSConnector'
    AWSCodeDeployRoleForECSLimited = 'arn:aws:iam::aws:policy/AWSCodeDeployRoleForECSLimited'
    AmazonElasticTranscoder_JobsSubmitter = 'arn:aws:iam::aws:policy/AmazonElasticTranscoder_JobsSubmitter'
    AmazonMacieHandshakeRole = 'arn:aws:iam::aws:policy/service-role/AmazonMacieHandshakeRole'
    AWSIoTAnalyticsFullAccess = 'arn:aws:iam::aws:policy/AWSIoTAnalyticsFullAccess'
    AWSBatchFullAccess = 'arn:aws:iam::aws:policy/AWSBatchFullAccess'
    AmazonSSMDirectoryServiceAccess = 'arn:aws:iam::aws:policy/AmazonSSMDirectoryServiceAccess'
    AmazonECS_FullAccess = 'arn:aws:iam::aws:policy/AmazonECS_FullAccess'
    AWSSupportServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSSupportServiceRolePolicy'
    AWSApplicationAutoscalingRDSClusterPolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSApplicationAutoscalingRDSClusterPolicy'
    AWSServiceRoleForEC2ScheduledInstances = 'arn:aws:iam::aws:policy/aws-service-role/AWSServiceRoleForEC2ScheduledInstances'
    AWSCodeDeployRoleForLambda = 'arn:aws:iam::aws:policy/service-role/AWSCodeDeployRoleForLambda'
    AWSFMAdminReadOnlyAccess = 'arn:aws:iam::aws:policy/AWSFMAdminReadOnlyAccess'
    AmazonSSMFullAccess = 'arn:aws:iam::aws:policy/AmazonSSMFullAccess'
    AWSCodeCommitReadOnly = 'arn:aws:iam::aws:policy/AWSCodeCommitReadOnly'
    AmazonEC2ContainerServiceFullAccess = 'arn:aws:iam::aws:policy/AmazonEC2ContainerServiceFullAccess'
    AmazonFreeRTOSFullAccess = 'arn:aws:iam::aws:policy/AmazonFreeRTOSFullAccess'
    AmazonTextractServiceRole = 'arn:aws:iam::aws:policy/service-role/AmazonTextractServiceRole'
    AmazonCognitoReadOnly = 'arn:aws:iam::aws:policy/AmazonCognitoReadOnly'
    AmazonDMSCloudWatchLogsRole = 'arn:aws:iam::aws:policy/service-role/AmazonDMSCloudWatchLogsRole'
    AWSApplicationDiscoveryServiceFullAccess = 'arn:aws:iam::aws:policy/AWSApplicationDiscoveryServiceFullAccess'
    AmazonRoute53AutoNamingReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonRoute53AutoNamingReadOnlyAccess'
    AWSSSOReadOnly = 'arn:aws:iam::aws:policy/AWSSSOReadOnly'
    AmazonVPCFullAccess = 'arn:aws:iam::aws:policy/AmazonVPCFullAccess'
    AWSCertificateManagerPrivateCAUser = 'arn:aws:iam::aws:policy/AWSCertificateManagerPrivateCAUser'
    AWSAppSyncAdministrator = 'arn:aws:iam::aws:policy/AWSAppSyncAdministrator'
    AWSEC2FleetServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSEC2FleetServiceRolePolicy'
    AmazonRoute53AutoNamingFullAccess = 'arn:aws:iam::aws:policy/AmazonRoute53AutoNamingFullAccess'
    AWSImportExportFullAccess = 'arn:aws:iam::aws:policy/AWSImportExportFullAccess'
    DynamoDBReplicationServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/DynamoDBReplicationServiceRolePolicy'
    AmazonMechanicalTurkFullAccess = 'arn:aws:iam::aws:policy/AmazonMechanicalTurkFullAccess'
    AmazonEC2ContainerRegistryPowerUser = 'arn:aws:iam::aws:policy/AmazonEC2ContainerRegistryPowerUser'
    AWSSSODirectoryReadOnly = 'arn:aws:iam::aws:policy/AWSSSODirectoryReadOnly'
    AmazonMachineLearningCreateOnlyAccess = 'arn:aws:iam::aws:policy/AmazonMachineLearningCreateOnlyAccess'
    AmazonKinesisVideoStreamsReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonKinesisVideoStreamsReadOnlyAccess'
    AWSCloudTrailReadOnlyAccess = 'arn:aws:iam::aws:policy/AWSCloudTrailReadOnlyAccess'
    WAFRegionalLoggingServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/WAFRegionalLoggingServiceRolePolicy'
    AWSLambdaExecute = 'arn:aws:iam::aws:policy/AWSLambdaExecute'
    AWSGlueConsoleSageMakerNotebookFullAccess = 'arn:aws:iam::aws:policy/AWSGlueConsoleSageMakerNotebookFullAccess'
    AmazonMSKFullAccess = 'arn:aws:iam::aws:policy/AmazonMSKFullAccess'
    AWSIoTRuleActions = 'arn:aws:iam::aws:policy/service-role/AWSIoTRuleActions'
    AmazonEKSServicePolicy = 'arn:aws:iam::aws:policy/AmazonEKSServicePolicy'
    AWSQuickSightDescribeRedshift = 'arn:aws:iam::aws:policy/service-role/AWSQuickSightDescribeRedshift'
    AmazonElasticsearchServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AmazonElasticsearchServiceRolePolicy'
    AmazonMQReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonMQReadOnlyAccess'
    VMImportExportRoleForAWSConnector = 'arn:aws:iam::aws:policy/service-role/VMImportExportRoleForAWSConnector'
    AWSCodePipelineCustomActionAccess = 'arn:aws:iam::aws:policy/AWSCodePipelineCustomActionAccess'
    AWSLambdaSQSQueueExecutionRole = 'arn:aws:iam::aws:policy/service-role/AWSLambdaSQSQueueExecutionRole'
    AWSCloud9ServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSCloud9ServiceRolePolicy'
    AWSApplicationAutoscalingECSServicePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSApplicationAutoscalingECSServicePolicy'
    AWSOpsWorksInstanceRegistration = 'arn:aws:iam::aws:policy/AWSOpsWorksInstanceRegistration'
    AmazonCloudDirectoryFullAccess = 'arn:aws:iam::aws:policy/AmazonCloudDirectoryFullAccess'
    AmazonECSTaskExecutionRolePolicy = 'arn:aws:iam::aws:policy/service-role/AmazonECSTaskExecutionRolePolicy'
    AWSStorageGatewayFullAccess = 'arn:aws:iam::aws:policy/AWSStorageGatewayFullAccess'
    AWSIoTEventsFullAccess = 'arn:aws:iam::aws:policy/AWSIoTEventsFullAccess'
    AmazonLexReadOnly = 'arn:aws:iam::aws:policy/AmazonLexReadOnly'
    TagPoliciesServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/TagPoliciesServiceRolePolicy'
    AmazonChimeUserManagement = 'arn:aws:iam::aws:policy/AmazonChimeUserManagement'
    AmazonMSKReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonMSKReadOnlyAccess'
    AWSDataSyncFullAccess = 'arn:aws:iam::aws:policy/AWSDataSyncFullAccess'
    AWSServiceRoleForIoTSiteWise = 'arn:aws:iam::aws:policy/aws-service-role/AWSServiceRoleForIoTSiteWise'
    CloudwatchApplicationInsightsServiceLinkedRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/CloudwatchApplicationInsightsServiceLinkedRolePolicy'
    AWSTrustedAdvisorServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSTrustedAdvisorServiceRolePolicy'
    AWSIoTConfigReadOnlyAccess = 'arn:aws:iam::aws:policy/AWSIoTConfigReadOnlyAccess'
    AmazonWorkMailReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonWorkMailReadOnlyAccess'
    AmazonDMSVPCManagementRole = 'arn:aws:iam::aws:policy/service-role/AmazonDMSVPCManagementRole'
    AWSLambdaKinesisExecutionRole = 'arn:aws:iam::aws:policy/service-role/AWSLambdaKinesisExecutionRole'
    ComprehendDataAccessRolePolicy = 'arn:aws:iam::aws:policy/service-role/ComprehendDataAccessRolePolicy'
    AmazonDocDBConsoleFullAccess = 'arn:aws:iam::aws:policy/AmazonDocDBConsoleFullAccess'
    ResourceGroupsandTagEditorReadOnlyAccess = 'arn:aws:iam::aws:policy/ResourceGroupsandTagEditorReadOnlyAccess'
    AmazonRekognitionServiceRole = 'arn:aws:iam::aws:policy/service-role/AmazonRekognitionServiceRole'
    AmazonSSMAutomationRole = 'arn:aws:iam::aws:policy/service-role/AmazonSSMAutomationRole'
    CloudHSMServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/CloudHSMServiceRolePolicy'
    ComprehendReadOnly = 'arn:aws:iam::aws:policy/ComprehendReadOnly'
    AWSStepFunctionsConsoleFullAccess = 'arn:aws:iam::aws:policy/AWSStepFunctionsConsoleFullAccess'
    AWSQuickSightIoTAnalyticsAccess = 'arn:aws:iam::aws:policy/AWSQuickSightIoTAnalyticsAccess'
    AWSCodeBuildReadOnlyAccess = 'arn:aws:iam::aws:policy/AWSCodeBuildReadOnlyAccess'
    LexBotPolicy = 'arn:aws:iam::aws:policy/aws-service-role/LexBotPolicy'
    AmazonMacieFullAccess = 'arn:aws:iam::aws:policy/AmazonMacieFullAccess'
    AmazonMachineLearningManageRealTimeEndpointOnlyAccess = 'arn:aws:iam::aws:policy/AmazonMachineLearningManageRealTimeEndpointOnlyAccess'
    CloudWatchEventsInvocationAccess = 'arn:aws:iam::aws:policy/service-role/CloudWatchEventsInvocationAccess'
    CloudFrontReadOnlyAccess = 'arn:aws:iam::aws:policy/CloudFrontReadOnlyAccess'
    AWSDeepLensServiceRolePolicy = 'arn:aws:iam::aws:policy/service-role/AWSDeepLensServiceRolePolicy'
    AmazonSNSRole = 'arn:aws:iam::aws:policy/service-role/AmazonSNSRole'
    AmazonInspectorServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AmazonInspectorServiceRolePolicy'
    AmazonMobileAnalyticsFinancialReportAccess = 'arn:aws:iam::aws:policy/AmazonMobileAnalyticsFinancialReportAccess'
    AWSElasticBeanstalkService = 'arn:aws:iam::aws:policy/service-role/AWSElasticBeanstalkService'
    IAMReadOnlyAccess = 'arn:aws:iam::aws:policy/IAMReadOnlyAccess'
    AmazonRDSReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonRDSReadOnlyAccess'
    AWSIoTDeviceDefenderAudit = 'arn:aws:iam::aws:policy/service-role/AWSIoTDeviceDefenderAudit'
    AmazonCognitoPowerUser = 'arn:aws:iam::aws:policy/AmazonCognitoPowerUser'
    AmazonRoute53AutoNamingRegistrantAccess = 'arn:aws:iam::aws:policy/AmazonRoute53AutoNamingRegistrantAccess'
    AmazonElasticFileSystemFullAccess = 'arn:aws:iam::aws:policy/AmazonElasticFileSystemFullAccess'
    LexChannelPolicy = 'arn:aws:iam::aws:policy/aws-service-role/LexChannelPolicy'
    ServerMigrationConnector = 'arn:aws:iam::aws:policy/ServerMigrationConnector'
    AmazonESCognitoAccess = 'arn:aws:iam::aws:policy/AmazonESCognitoAccess'
    AWSFMAdminFullAccess = 'arn:aws:iam::aws:policy/AWSFMAdminFullAccess'
    AmazonChimeReadOnly = 'arn:aws:iam::aws:policy/AmazonChimeReadOnly'
    AmazonZocaloFullAccess = 'arn:aws:iam::aws:policy/AmazonZocaloFullAccess'
    AWSLambdaReadOnlyAccess = 'arn:aws:iam::aws:policy/AWSLambdaReadOnlyAccess'
    AWSIoTSiteWiseReadOnlyAccess = 'arn:aws:iam::aws:policy/AWSIoTSiteWiseReadOnlyAccess'
    AWSAccountUsageReportAccess = 'arn:aws:iam::aws:policy/AWSAccountUsageReportAccess'
    AWSIoTOTAUpdate = 'arn:aws:iam::aws:policy/service-role/AWSIoTOTAUpdate'
    AmazonMQFullAccess = 'arn:aws:iam::aws:policy/AmazonMQFullAccess'
    AWSMarketplaceGetEntitlements = 'arn:aws:iam::aws:policy/AWSMarketplaceGetEntitlements'
    AWSGreengrassReadOnlyAccess = 'arn:aws:iam::aws:policy/AWSGreengrassReadOnlyAccess'
    AmazonEC2ContainerServiceforEC2Role = 'arn:aws:iam::aws:policy/service-role/AmazonEC2ContainerServiceforEC2Role'
    AmazonAppStreamFullAccess = 'arn:aws:iam::aws:policy/AmazonAppStreamFullAccess'
    AWSIoTDataAccess = 'arn:aws:iam::aws:policy/AWSIoTDataAccess'
    AmazonWorkLinkFullAccess = 'arn:aws:iam::aws:policy/AmazonWorkLinkFullAccess'
    AmazonTranscribeReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonTranscribeReadOnlyAccess'
    AmazonESFullAccess = 'arn:aws:iam::aws:policy/AmazonESFullAccess'
    ServerMigrationServiceRole = 'arn:aws:iam::aws:policy/service-role/ServerMigrationServiceRole'
    ApplicationDiscoveryServiceContinuousExportServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/ApplicationDiscoveryServiceContinuousExportServiceRolePolicy'
    AmazonSumerianFullAccess = 'arn:aws:iam::aws:policy/AmazonSumerianFullAccess'
    AWSWAFFullAccess = 'arn:aws:iam::aws:policy/AWSWAFFullAccess'
    ElasticLoadBalancingReadOnly = 'arn:aws:iam::aws:policy/ElasticLoadBalancingReadOnly'
    AWSArtifactAccountSync = 'arn:aws:iam::aws:policy/service-role/AWSArtifactAccountSync'
    AmazonKinesisFirehoseFullAccess = 'arn:aws:iam::aws:policy/AmazonKinesisFirehoseFullAccess'
    CloudWatchReadOnlyAccess = 'arn:aws:iam::aws:policy/CloudWatchReadOnlyAccess'
    AWSLambdaBasicExecutionRole = 'arn:aws:iam::aws:policy/service-role/AWSLambdaBasicExecutionRole'
    ResourceGroupsandTagEditorFullAccess = 'arn:aws:iam::aws:policy/ResourceGroupsandTagEditorFullAccess'
    AWSKeyManagementServicePowerUser = 'arn:aws:iam::aws:policy/AWSKeyManagementServicePowerUser'
    AWSApplicationAutoscalingEC2SpotFleetRequestPolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSApplicationAutoscalingEC2SpotFleetRequestPolicy'
    AWSImportExportReadOnlyAccess = 'arn:aws:iam::aws:policy/AWSImportExportReadOnlyAccess'
    CloudWatchEventsServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/CloudWatchEventsServiceRolePolicy'
    AmazonElasticTranscoderRole = 'arn:aws:iam::aws:policy/service-role/AmazonElasticTranscoderRole'
    AWSGlueConsoleFullAccess = 'arn:aws:iam::aws:policy/AWSGlueConsoleFullAccess'
    AmazonEC2ContainerServiceRole = 'arn:aws:iam::aws:policy/service-role/AmazonEC2ContainerServiceRole'
    AWSDeviceFarmFullAccess = 'arn:aws:iam::aws:policy/AWSDeviceFarmFullAccess'
    AmazonSSMReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonSSMReadOnlyAccess'
    AWSStepFunctionsReadOnlyAccess = 'arn:aws:iam::aws:policy/AWSStepFunctionsReadOnlyAccess'
    AWSMarketplaceReadonly = 'arn:aws:iam::aws:policy/AWSMarketplaceRead-only'
    AWSApplicationAutoscalingDynamoDBTablePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSApplicationAutoscalingDynamoDBTablePolicy'
    AWSCodePipelineFullAccess = 'arn:aws:iam::aws:policy/AWSCodePipelineFullAccess'
    AWSCloud9User = 'arn:aws:iam::aws:policy/AWSCloud9User'
    AWSGreengrassResourceAccessRolePolicy = 'arn:aws:iam::aws:policy/service-role/AWSGreengrassResourceAccessRolePolicy'
    AmazonMacieServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AmazonMacieServiceRolePolicy'
    NetworkAdministrator = 'arn:aws:iam::aws:policy/job-function/NetworkAdministrator'
    AWSIoT1ClickFullAccess = 'arn:aws:iam::aws:policy/AWSIoT1ClickFullAccess'
    AmazonWorkSpacesApplicationManagerAdminAccess = 'arn:aws:iam::aws:policy/AmazonWorkSpacesApplicationManagerAdminAccess'
    AmazonDRSVPCManagement = 'arn:aws:iam::aws:policy/AmazonDRSVPCManagement'
    AmazonRedshiftServiceLinkedRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AmazonRedshiftServiceLinkedRolePolicy'
    AWSCertificateManagerPrivateCAReadOnly = 'arn:aws:iam::aws:policy/AWSCertificateManagerPrivateCAReadOnly'
    AWSXrayFullAccess = 'arn:aws:iam::aws:policy/AWSXrayFullAccess'
    AWSElasticBeanstalkWorkerTier = 'arn:aws:iam::aws:policy/AWSElasticBeanstalkWorkerTier'
    AWSDirectConnectFullAccess = 'arn:aws:iam::aws:policy/AWSDirectConnectFullAccess'
    AWSCodeBuildAdminAccess = 'arn:aws:iam::aws:policy/AWSCodeBuildAdminAccess'
    AmazonKinesisAnalyticsFullAccess = 'arn:aws:iam::aws:policy/AmazonKinesisAnalyticsFullAccess'
    AWSSecurityHubServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSSecurityHubServiceRolePolicy'
    AWSElasticBeanstalkMaintenance = 'arn:aws:iam::aws:policy/aws-service-role/AWSElasticBeanstalkMaintenance'
    APIGatewayServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/APIGatewayServiceRolePolicy'
    AWSAccountActivityAccess = 'arn:aws:iam::aws:policy/AWSAccountActivityAccess'
    AmazonGlacierFullAccess = 'arn:aws:iam::aws:policy/AmazonGlacierFullAccess'
    AmazonFSxConsoleReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonFSxConsoleReadOnlyAccess'
    AmazonWorkMailFullAccess = 'arn:aws:iam::aws:policy/AmazonWorkMailFullAccess'
    DAXServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/DAXServiceRolePolicy'
    ComprehendMedicalFullAccess = 'arn:aws:iam::aws:policy/ComprehendMedicalFullAccess'
    AWSMarketplaceManageSubscriptions = 'arn:aws:iam::aws:policy/AWSMarketplaceManageSubscriptions'
    AWSElasticBeanstalkCustomPlatformforEC2Role = 'arn:aws:iam::aws:policy/AWSElasticBeanstalkCustomPlatformforEC2Role'
    AWSDataSyncReadOnlyAccess = 'arn:aws:iam::aws:policy/AWSDataSyncReadOnlyAccess'
    AWSVPCTransitGatewayServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSVPCTransitGatewayServiceRolePolicy'
    NeptuneReadOnlyAccess = 'arn:aws:iam::aws:policy/NeptuneReadOnlyAccess'
    AWSSupportAccess = 'arn:aws:iam::aws:policy/AWSSupportAccess'
    AmazonElasticMapReduceforAutoScalingRole = 'arn:aws:iam::aws:policy/service-role/AmazonElasticMapReduceforAutoScalingRole'
    AWSElementalMediaConvertReadOnly = 'arn:aws:iam::aws:policy/AWSElementalMediaConvertReadOnly'
    AWSLambdaInvocationDynamoDB = 'arn:aws:iam::aws:policy/AWSLambdaInvocation-DynamoDB'
    AWSServiceCatalogEndUserFullAccess = 'arn:aws:iam::aws:policy/AWSServiceCatalogEndUserFullAccess'
    IAMUserSSHKeys = 'arn:aws:iam::aws:policy/IAMUserSSHKeys'
    AWSDeepRacerServiceRolePolicy = 'arn:aws:iam::aws:policy/service-role/AWSDeepRacerServiceRolePolicy'
    AmazonSageMakerReadOnly = 'arn:aws:iam::aws:policy/AmazonSageMakerReadOnly'
    AWSIoTFullAccess = 'arn:aws:iam::aws:policy/AWSIoTFullAccess'
    AWSQuickSightDescribeRDS = 'arn:aws:iam::aws:policy/service-role/AWSQuickSightDescribeRDS'
    AWSResourceAccessManagerServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSResourceAccessManagerServiceRolePolicy'
    AWSConfigRulesExecutionRole = 'arn:aws:iam::aws:policy/service-role/AWSConfigRulesExecutionRole'
    AWSConfigServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSConfigServiceRolePolicy'
    AmazonESReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonESReadOnlyAccess'
    AWSCodeDeployDeployerAccess = 'arn:aws:iam::aws:policy/AWSCodeDeployDeployerAccess'
    KafkaServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/KafkaServiceRolePolicy'
    AmazonPollyFullAccess = 'arn:aws:iam::aws:policy/AmazonPollyFullAccess'
    AmazonSSMMaintenanceWindowRole = 'arn:aws:iam::aws:policy/service-role/AmazonSSMMaintenanceWindowRole'
    AmazonRDSEnhancedMonitoringRole = 'arn:aws:iam::aws:policy/service-role/AmazonRDSEnhancedMonitoringRole'
    AmazonLexFullAccess = 'arn:aws:iam::aws:policy/AmazonLexFullAccess'
    AWSLambdaVPCAccessExecutionRole = 'arn:aws:iam::aws:policy/service-role/AWSLambdaVPCAccessExecutionRole'
    AmazonMacieServiceRole = 'arn:aws:iam::aws:policy/service-role/AmazonMacieServiceRole'
    AmazonLexRunBotsOnly = 'arn:aws:iam::aws:policy/AmazonLexRunBotsOnly'
    AWSCertificateManagerPrivateCAAuditor = 'arn:aws:iam::aws:policy/AWSCertificateManagerPrivateCAAuditor'
    AmazonSNSFullAccess = 'arn:aws:iam::aws:policy/AmazonSNSFullAccess'
    AmazonEKS_CNI_Policy = 'arn:aws:iam::aws:policy/AmazonEKS_CNI_Policy'
    AWSServiceCatalogAdminFullAccess = 'arn:aws:iam::aws:policy/AWSServiceCatalogAdminFullAccess'
    AWSShieldDRTAccessPolicy = 'arn:aws:iam::aws:policy/service-role/AWSShieldDRTAccessPolicy'
    CloudSearchReadOnlyAccess = 'arn:aws:iam::aws:policy/CloudSearchReadOnlyAccess'
    AWSGreengrassFullAccess = 'arn:aws:iam::aws:policy/AWSGreengrassFullAccess'
    NeptuneConsoleFullAccess = 'arn:aws:iam::aws:policy/NeptuneConsoleFullAccess'
    AWSCloudFormationReadOnlyAccess = 'arn:aws:iam::aws:policy/AWSCloudFormationReadOnlyAccess'
    AmazonRoute53FullAccess = 'arn:aws:iam::aws:policy/AmazonRoute53FullAccess'
    AWSLambdaRole = 'arn:aws:iam::aws:policy/service-role/AWSLambdaRole'
    AWSLambdaENIManagementAccess = 'arn:aws:iam::aws:policy/service-role/AWSLambdaENIManagementAccess'
    AWSOpsWorksCloudWatchLogs = 'arn:aws:iam::aws:policy/AWSOpsWorksCloudWatchLogs'
    AmazonAppStreamReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonAppStreamReadOnlyAccess'
    AWSStepFunctionsFullAccess = 'arn:aws:iam::aws:policy/AWSStepFunctionsFullAccess'
    CloudTrailServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/CloudTrailServiceRolePolicy'
    AmazonInspectorReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonInspectorReadOnlyAccess'
    AWSOrganizationsReadOnlyAccess = 'arn:aws:iam::aws:policy/AWSOrganizationsReadOnlyAccess'
    TranslateReadOnly = 'arn:aws:iam::aws:policy/TranslateReadOnly'
    AWSCertificateManagerFullAccess = 'arn:aws:iam::aws:policy/AWSCertificateManagerFullAccess'
    AWSDeepRacerCloudFormationAccessPolicy = 'arn:aws:iam::aws:policy/AWSDeepRacerCloudFormationAccessPolicy'
    AWSIoTEventsReadOnlyAccess = 'arn:aws:iam::aws:policy/AWSIoTEventsReadOnlyAccess'
    AWSRoboMakerServicePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSRoboMakerServicePolicy'
    PowerUserAccess = 'arn:aws:iam::aws:policy/PowerUserAccess'
    AWSApplicationAutoScalingCustomResourcePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSApplicationAutoScalingCustomResourcePolicy'
    GlobalAcceleratorReadOnlyAccess = 'arn:aws:iam::aws:policy/GlobalAcceleratorReadOnlyAccess'
    AmazonSageMakerFullAccess = 'arn:aws:iam::aws:policy/AmazonSageMakerFullAccess'
    WAFLoggingServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/WAFLoggingServiceRolePolicy'
    AWSBackupServiceRolePolicyForRestores = 'arn:aws:iam::aws:policy/service-role/AWSBackupServiceRolePolicyForRestores'
    AWSElementalMediaStoreFullAccess = 'arn:aws:iam::aws:policy/AWSElementalMediaStoreFullAccess'
    CloudWatchEventsFullAccess = 'arn:aws:iam::aws:policy/CloudWatchEventsFullAccess'
    AWSLicenseManagerMemberAccountRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSLicenseManagerMemberAccountRolePolicy'
    AWSOrganizationsFullAccess = 'arn:aws:iam::aws:policy/AWSOrganizationsFullAccess'
    AmazonFraudDetectorFullAccessPolicy = 'arn:aws:iam::aws:policy/AmazonFraudDetectorFullAccessPolicy'
    AmazonChimeSDK = 'arn:aws:iam::aws:policy/AmazonChimeSDK'
    AWSIoTDeviceTesterForFreeRTOSFullAccess = 'arn:aws:iam::aws:policy/AWSIoTDeviceTesterForFreeRTOSFullAccess'
    WAFV2LoggingServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/WAFV2LoggingServiceRolePolicy'
    AWSNetworkManagerFullAccess = 'arn:aws:iam::aws:policy/AWSNetworkManagerFullAccess'
    AWSPrivateMarketplaceRequests = 'arn:aws:iam::aws:policy/AWSPrivateMarketplaceRequests'
    AmazonSageMakerMechanicalTurkAccess = 'arn:aws:iam::aws:policy/AmazonSageMakerMechanicalTurkAccess'
    AWSNetworkManagerServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSNetworkManagerServiceRolePolicy'
    AWSAppMeshServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSAppMeshServiceRolePolicy'
    AWSConfigRemediationServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSConfigRemediationServiceRolePolicy'
    ConfigConformsServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/ConfigConformsServiceRolePolicy'
    AmazonEventBridgeReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonEventBridgeReadOnlyAccess'
    AWSCodeStarNotificationsServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSCodeStarNotificationsServiceRolePolicy'
    AmazonKendraFullAccess = 'arn:aws:iam::aws:policy/AmazonKendraFullAccess'
    AWSSystemsManagerAccountDiscoveryServicePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSSystemsManagerAccountDiscoveryServicePolicy'
    AWSResourceAccessManagerReadOnlyAccess = 'arn:aws:iam::aws:policy/AWSResourceAccessManagerReadOnlyAccess'
    AmazonEventBridgeFullAccess = 'arn:aws:iam::aws:policy/AmazonEventBridgeFullAccess'
    CloudWatchSyntheticsReadOnlyAccess = 'arn:aws:iam::aws:policy/CloudWatchSyntheticsReadOnlyAccess'
    AccessAnalyzerServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AccessAnalyzerServiceRolePolicy'
    AmazonRoute53ResolverReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonRoute53ResolverReadOnlyAccess'
    AmazonEC2RolePolicyForLaunchWizard = 'arn:aws:iam::aws:policy/AmazonEC2RolePolicyForLaunchWizard'
    AmazonManagedBlockchainFullAccess = 'arn:aws:iam::aws:policy/AmazonManagedBlockchainFullAccess'
    ServiceQuotasFullAccess = 'arn:aws:iam::aws:policy/ServiceQuotasFullAccess'
    AWSIoTSiteWiseMonitorServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSIoTSiteWiseMonitorServiceRolePolicy'
    AWSCloudFormationFullAccess = 'arn:aws:iam::aws:policy/AWSCloudFormationFullAccess'
    ElementalAppliancesSoftwareFullAccess = 'arn:aws:iam::aws:policy/ElementalAppliancesSoftwareFullAccess'
    AmazonAugmentedAIHumanLoopFullAccess = 'arn:aws:iam::aws:policy/AmazonAugmentedAIHumanLoopFullAccess'
    AWSDataExchangeReadOnly = 'arn:aws:iam::aws:policy/AWSDataExchangeReadOnly'
    AWSMarketplaceSellerProductsFullAccess = 'arn:aws:iam::aws:policy/AWSMarketplaceSellerProductsFullAccess'
    AWSIQContractServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSIQContractServiceRolePolicy'
    AmazonLaunchWizardFullaccess = 'arn:aws:iam::aws:policy/AmazonLaunchWizardFullaccess'
    AmazonWorkDocsReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonWorkDocsReadOnlyAccess'
    AWSGlobalAcceleratorSLRPolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSGlobalAcceleratorSLRPolicy'
    EC2InstanceProfileForImageBuilder = 'arn:aws:iam::aws:policy/EC2InstanceProfileForImageBuilder'
    AWSServiceRoleForLogDeliveryPolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSServiceRoleForLogDeliveryPolicy'
    AmazonCodeGuruReviewerFullAccess = 'arn:aws:iam::aws:policy/AmazonCodeGuruReviewerFullAccess'
    AWSVPCS2SVpnServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSVPCS2SVpnServiceRolePolicy'
    AWSImageBuilderFullAccess = 'arn:aws:iam::aws:policy/AWSImageBuilderFullAccess'
    AWSCertificateManagerPrivateCAPrivilegedUser = 'arn:aws:iam::aws:policy/AWSCertificateManagerPrivateCAPrivilegedUser'
    AWSOpsWorksRegisterCLI_OnPremises = 'arn:aws:iam::aws:policy/AWSOpsWorksRegisterCLI_OnPremises'
    Health_OrganizationsServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/Health_OrganizationsServiceRolePolicy'
    AmazonMCSReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonMCSReadOnlyAccess'
    AWSAppMeshPreviewServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSAppMeshPreviewServiceRolePolicy'
    ServiceQuotasServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/ServiceQuotasServiceRolePolicy'
    ComputeOptimizerReadOnlyAccess = 'arn:aws:iam::aws:policy/ComputeOptimizerReadOnlyAccess'
    AlexaForBusinessPolyDelegatedAccessPolicy = 'arn:aws:iam::aws:policy/AlexaForBusinessPolyDelegatedAccessPolicy'
    AWSMarketplaceProcurementSystemAdminFullAccess = 'arn:aws:iam::aws:policy/AWSMarketplaceProcurementSystemAdminFullAccess'
    AmazonEKSFargatePodExecutionRolePolicy = 'arn:aws:iam::aws:policy/AmazonEKSFargatePodExecutionRolePolicy'
    IAMAccessAdvisorReadOnly = 'arn:aws:iam::aws:policy/IAMAccessAdvisorReadOnly'
    AmazonCodeGuruReviewerReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonCodeGuruReviewerReadOnlyAccess'
    AmazonCodeGuruProfilerFullAccess = 'arn:aws:iam::aws:policy/AmazonCodeGuruProfilerFullAccess'
    AmazonElasticFileSystemServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AmazonElasticFileSystemServiceRolePolicy'
    AWSResourceAccessManagerFullAccess = 'arn:aws:iam::aws:policy/AWSResourceAccessManagerFullAccess'
    AWSIoTDeviceDefenderEnableIoTLoggingMitigationAction = 'arn:aws:iam::aws:policy/service-role/AWSIoTDeviceDefenderEnableIoTLoggingMitigationAction'
    DynamoDBCloudWatchContributorInsightsServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/DynamoDBCloudWatchContributorInsightsServiceRolePolicy'
    AmazonChimeVoiceConnectorServiceLinkedRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AmazonChimeVoiceConnectorServiceLinkedRolePolicy'
    IAMAccessAnalyzerReadOnlyAccess = 'arn:aws:iam::aws:policy/IAMAccessAnalyzerReadOnlyAccess'
    AmazonEventBridgeSchemasServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AmazonEventBridgeSchemasServiceRolePolicy'
    AWSIoTDeviceDefenderPublishFindingsToSNSMitigationAction = 'arn:aws:iam::aws:policy/service-role/AWSIoTDeviceDefenderPublishFindingsToSNSMitigationAction'
    AmazonQLDBConsoleFullAccess = 'arn:aws:iam::aws:policy/AmazonQLDBConsoleFullAccess'
    AmazonElasticFileSystemClientReadWriteAccess = 'arn:aws:iam::aws:policy/AmazonElasticFileSystemClientReadWriteAccess'
    AWSApplicationAutoscalingComprehendEndpointPolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSApplicationAutoscalingComprehendEndpointPolicy'
    AWSIoTDeviceDefenderAddThingsToThingGroupMitigationAction = 'arn:aws:iam::aws:policy/service-role/AWSIoTDeviceDefenderAddThingsToThingGroupMitigationAction'
    AmazonQLDBFullAccess = 'arn:aws:iam::aws:policy/AmazonQLDBFullAccess'
    AmazonAugmentedAIFullAccess = 'arn:aws:iam::aws:policy/AmazonAugmentedAIFullAccess'
    AWSIoTDeviceDefenderReplaceDefaultPolicyMitigationAction = 'arn:aws:iam::aws:policy/service-role/AWSIoTDeviceDefenderReplaceDefaultPolicyMitigationAction'
    AWSAppMeshReadOnly = 'arn:aws:iam::aws:policy/AWSAppMeshReadOnly'
    ComputeOptimizerServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/ComputeOptimizerServiceRolePolicy'
    AWSElasticBeanstalkManagedUpdatesServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSElasticBeanstalkManagedUpdatesServiceRolePolicy'
    AmazonQLDBReadOnly = 'arn:aws:iam::aws:policy/AmazonQLDBReadOnly'
    AWSChatbotServiceLinkedRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSChatbotServiceLinkedRolePolicy'
    AWSAppSyncServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSAppSyncServiceRolePolicy'
    AWSAppMeshFullAccess = 'arn:aws:iam::aws:policy/AWSAppMeshFullAccess'
    AWSServiceRoleForGammaInternalAmazonEKSNodegroup = 'arn:aws:iam::aws:policy/AWSServiceRoleForGammaInternalAmazonEKSNodegroup'
    ServiceQuotasReadOnlyAccess = 'arn:aws:iam::aws:policy/ServiceQuotasReadOnlyAccess'
    EC2FleetTimeShiftableServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/EC2FleetTimeShiftableServiceRolePolicy'
    MigrationHubDMSAccessServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/MigrationHubDMSAccessServiceRolePolicy'
    AWSServiceCatalogEndUserReadOnlyAccess = 'arn:aws:iam::aws:policy/AWSServiceCatalogEndUserReadOnlyAccess'
    AWSIQPermissionServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSIQPermissionServiceRolePolicy'
    AmazonEKSForFargateServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AmazonEKSForFargateServiceRolePolicy'
    MigrationHubSMSAccessServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/MigrationHubSMSAccessServiceRolePolicy'
    CloudFormationStackSetsOrgAdminServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/CloudFormationStackSetsOrgAdminServiceRolePolicy'
    AmazonEventBridgeSchemasFullAccess = 'arn:aws:iam::aws:policy/AmazonEventBridgeSchemasFullAccess'
    AWSMarketplaceSellerFullAccess = 'arn:aws:iam::aws:policy/AWSMarketplaceSellerFullAccess'
    CloudWatchAutomaticDashboardsAccess = 'arn:aws:iam::aws:policy/CloudWatchAutomaticDashboardsAccess'
    AmazonWorkMailEventsServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AmazonWorkMailEventsServiceRolePolicy'
    AmazonEventBridgeSchemasReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonEventBridgeSchemasReadOnlyAccess'
    AWSMarketplaceSellerProductsReadOnly = 'arn:aws:iam::aws:policy/AWSMarketplaceSellerProductsReadOnly'
    AmazonMCSFullAccess = 'arn:aws:iam::aws:policy/AmazonMCSFullAccess'
    AWSIoTSiteWiseConsoleFullAccess = 'arn:aws:iam::aws:policy/AWSIoTSiteWiseConsoleFullAccess'
    AmazonElasticFileSystemClientFullAccess = 'arn:aws:iam::aws:policy/AmazonElasticFileSystemClientFullAccess'
    AWSIoTDeviceDefenderUpdateDeviceCertMitigationAction = 'arn:aws:iam::aws:policy/service-role/AWSIoTDeviceDefenderUpdateDeviceCertMitigationAction'
    AWSForWordPressPluginPolicy = 'arn:aws:iam::aws:policy/AWSForWordPressPluginPolicy'
    AWSServiceRoleForAmazonEKSNodegroup = 'arn:aws:iam::aws:policy/aws-service-role/AWSServiceRoleForAmazonEKSNodegroup'
    AWSBackupOperatorAccess = 'arn:aws:iam::aws:policy/AWSBackupOperatorAccess'
    AWSApplicationAutoscalingLambdaConcurrencyPolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSApplicationAutoscalingLambdaConcurrencyPolicy'
    AmazonMachineLearningRoleforRedshiftDataSourceV2 = 'arn:aws:iam::aws:policy/service-role/AmazonMachineLearningRoleforRedshiftDataSourceV2'
    AWSIoTDeviceDefenderUpdateCACertMitigationAction = 'arn:aws:iam::aws:policy/service-role/AWSIoTDeviceDefenderUpdateCACertMitigationAction'
    AmazonWorkSpacesServiceAccess = 'arn:aws:iam::aws:policy/AmazonWorkSpacesServiceAccess'
    AmazonEKSServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AmazonEKSServiceRolePolicy'
    AWSConfigMultiAccountSetupPolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSConfigMultiAccountSetupPolicy'
    AmazonElasticFileSystemClientReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonElasticFileSystemClientReadOnlyAccess'
    CloudFormationStackSetsOrgMemberServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/CloudFormationStackSetsOrgMemberServiceRolePolicy'
    AWSResourceAccessManagerResourceShareParticipantAccess = 'arn:aws:iam::aws:policy/AWSResourceAccessManagerResourceShareParticipantAccess'
    AWSBackupFullAccess = 'arn:aws:iam::aws:policy/AWSBackupFullAccess'
    AmazonCodeGuruProfilerReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonCodeGuruProfilerReadOnlyAccess'
    AWSNetworkManagerReadOnlyAccess = 'arn:aws:iam::aws:policy/AWSNetworkManagerReadOnlyAccess'
    CloudWatchSyntheticsFullAccess = 'arn:aws:iam::aws:policy/CloudWatchSyntheticsFullAccess'
    AWSDataExchangeSubscriberFullAccess = 'arn:aws:iam::aws:policy/AWSDataExchangeSubscriberFullAccess'
    IAMAccessAnalyzerFullAccess = 'arn:aws:iam::aws:policy/IAMAccessAnalyzerFullAccess'
    AWSServiceCatalogAdminReadOnlyAccess = 'arn:aws:iam::aws:policy/AWSServiceCatalogAdminReadOnlyAccess'
    AWSQuickSightSageMakerPolicy = 'arn:aws:iam::aws:policy/service-role/AWSQuickSightSageMakerPolicy'
    AmazonWorkSpacesSelfServiceAccess = 'arn:aws:iam::aws:policy/AmazonWorkSpacesSelfServiceAccess'
    AmazonManagedBlockchainServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AmazonManagedBlockchainServiceRolePolicy'
    AWSDataExchangeFullAccess = 'arn:aws:iam::aws:policy/AWSDataExchangeFullAccess'
    AWSDataExchangeProviderFullAccess = 'arn:aws:iam::aws:policy/AWSDataExchangeProviderFullAccess'
    AWSControlTowerServiceRolePolicy = 'arn:aws:iam::aws:policy/service-role/AWSControlTowerServiceRolePolicy'
    AmazonSageMakerNotebooksServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AmazonSageMakerNotebooksServiceRolePolicy'
    AmazonRoute53ResolverFullAccess = 'arn:aws:iam::aws:policy/AmazonRoute53ResolverFullAccess'
    LakeFormationDataAccessServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/LakeFormationDataAccessServiceRolePolicy'
    AmazonChimeServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AmazonChimeServiceRolePolicy'
    AWSTrustedAdvisorReportingServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AWSTrustedAdvisorReportingServiceRolePolicy'
    AWSOpsWorksRegisterCLI_EC2 = 'arn:aws:iam::aws:policy/AWSOpsWorksRegisterCLI_EC2'
    AWSSavingsPlansFullAccess = 'arn:aws:iam::aws:policy/AWSSavingsPlansFullAccess'
    AWSServiceRoleForImageBuilder = 'arn:aws:iam::aws:policy/aws-service-role/AWSServiceRoleForImageBuilder'
    AmazonCodeGuruReviewerServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/AmazonCodeGuruReviewerServiceRolePolicy'
    AWSAppMeshPreviewEnvoyAccess = 'arn:aws:iam::aws:policy/AWSAppMeshPreviewEnvoyAccess'
    MigrationHubServiceRolePolicy = 'arn:aws:iam::aws:policy/aws-service-role/MigrationHubServiceRolePolicy'
    AWSImageBuilderReadOnlyAccess = 'arn:aws:iam::aws:policy/AWSImageBuilderReadOnlyAccess'
    AWSMarketplaceMeteringRegisterUsage = 'arn:aws:iam::aws:policy/AWSMarketplaceMeteringRegisterUsage'
    AmazonManagedBlockchainReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonManagedBlockchainReadOnlyAccess'
    AmazonRekognitionCustomLabelsFullAccess = 'arn:aws:iam::aws:policy/AmazonRekognitionCustomLabelsFullAccess'
    AmazonManagedBlockchainConsoleFullAccess = 'arn:aws:iam::aws:policy/AmazonManagedBlockchainConsoleFullAccess'
    AWSSavingsPlansReadOnlyAccess = 'arn:aws:iam::aws:policy/AWSSavingsPlansReadOnlyAccess'
    AWSIoTDeviceTesterForGreengrassFullAccess = 'arn:aws:iam::aws:policy/AWSIoTDeviceTesterForGreengrassFullAccess'
    AWSServiceRoleForSMS = 'arn:aws:iam::aws:policy/aws-service-role/AWSServiceRoleForSMS'
    CloudWatchCrossAccountAccess = 'arn:aws:iam::aws:policy/aws-service-role/CloudWatch-CrossAccountAccess'
    AWSLakeFormationDataAdmin = 'arn:aws:iam::aws:policy/AWSLakeFormationDataAdmin'
    AWSDenyAll = 'arn:aws:iam::aws:policy/AWSDenyAll'
    AWSIQFullAccess = 'arn:aws:iam::aws:policy/AWSIQFullAccess'
    EC2InstanceConnect = 'arn:aws:iam::aws:policy/EC2InstanceConnect'
    AWSAppMeshEnvoyAccess = 'arn:aws:iam::aws:policy/AWSAppMeshEnvoyAccess'
    AmazonKendraReadOnlyAccess = 'arn:aws:iam::aws:policy/AmazonKendraReadOnlyAccess'